# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource
from openstack import utils


class ShareInstance(resource.Resource):
    resource_key = "share_instance"
    resources_key = "share_instances"
    base_path = "/share_instances"

    # capabilities
    allow_create = False
    allow_fetch = True
    allow_commit = False
    allow_delete = False
    allow_list = True
    allow_head = False

    #: Properties
    #: The share instance access rules status. A valid value is active,
    #: error, or syncing.
    access_rules_status = resource.Body("access_rules_status", type=str)
    #: The name of the availability zone the share exists within.
    availability_zone = resource.Body("availability_zone", type=str)
    #: If the share instance has its cast_rules_to_readonly attribute
    #: set to True, all existing access rules be cast to read/only.
    cast_rules_to_readonly = resource.Body("cast_rules_to_readonly", type=bool)
    #: The date and time stamp when the resource was created within the
    #: services's database.
    created_at = resource.Body("created_at", type=str)
    #: The host name of the service back end that the resource is
    #: contained within.
    host = resource.Body("host", type=str)
    #: The progress of the share creation.
    progress = resource.Body("progress", type=str)
    #: The share replica state. Has set value only when replication is used.
    #: List of possible values: active, in_sync, out_of_sync, error
    replica_state = resource.Body("replica_state", type=str)
    #: The UUID of the share to which the share instance belongs to.
    share_id = resource.Body("share_id", type=str)
    #: The share network ID where the resource is exported to.
    share_network_id = resource.Body("share_network_id", type=str)
    #: The UUID of the share server.
    share_server_id = resource.Body("share_server_id", type=str)
    #: The share or share instance status.
    status = resource.Body("status", type=str)

    def _action(self, session, body, microversion=None):
        """Perform share instance actions given the message body"""
        url = utils.urljoin(self.base_path, self.id, 'action')
        headers = {'Accept': ''}
        extra_attrs = {}
        if microversion:
            # Set microversion override
            extra_attrs['microversion'] = microversion
        else:
            extra_attrs['microversion'] = self._get_microversion(session)
        response = session.post(url, json=body, headers=headers, **extra_attrs)
        exceptions.raise_from_response(response)
        return response

    def reset_status(self, session, reset_status):
        """Reset share instance to given status"""
        body = {"reset_status": {"status": reset_status}}
        self._action(session, body)

    def force_delete(self, session):
        """Force delete share instance"""
        body = {"force_delete": None}
        self._action(session, body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack import utils


class ShareGroupSnapshot(resource.Resource):
    resource_key = "share_group_snapshot"
    resources_key = "share_group_snapshots"
    base_path = "/share-group-snapshots"

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    allow_head = False

    _query_mapping = resource.QueryParameters(
        'project_id',
        'all_tenants',
        'name',
        'description',
        'status',
        'share_group_id',
        'limit',
        'offset',
        'sort_key',
        'sort_dir',
    )

    #: Properties
    #: The ID of the project that owns the resource.
    project_id = resource.Body("project_id", type=str)
    #: Filters by a share group snapshot status. A valid value is creating,
    #: error, available, deleting, error_deleting.
    status = resource.Body("status", type=str)
    #: The UUID of the share group.
    share_group_id = resource.Body("share_group_id", type=str)
    #: The user defined description of the resource.
    description = resource.Body("description", type=str)
    #: The date and time stamp when the resource was created.
    created_at = resource.Body("created_at", type=str)
    #: The share group snapshot members.
    members = resource.Body("members", type=str)
    #: The snapshot size, in GiBs.
    size = resource.Body("size", type=int)
    #: NFS, CIFS, GlusterFS, HDFS, CephFS or MAPRFS.
    share_protocol = resource.Body("share_proto", type=str)

    def _action(self, session, body, microversion=None):
        """Perform ShareGroupSnapshot actions given the message body."""
        # NOTE: This is using ShareGroupSnapshot.base_path instead of
        # self.base_path as ShareGroupSnapshot instances can be acted on,
        # but the URL used is sans any additional /detail/ part.
        url = utils.urljoin(self.base_path, self.id, 'action')
        headers = {'Accept': ''}
        microversion = microversion or self._get_microversion(session)
        extra_attrs = {'microversion': microversion}
        session.post(url, json=body, headers=headers, **extra_attrs)

    def reset_status(self, session, status):
        body = {"reset_status": {"status": status}}
        self._action(session, body)

    def get_members(self, session, microversion=None):
        url = utils.urljoin(self.base_path, self.id, 'members')
        microversion = microversion or self._get_microversion(session)
        headers = {'Accept': ''}
        response = session.get(url, headers=headers, microversion=microversion)
        return response.json()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import typing as ty

from openstack.accelerator.v2 import accelerator_request as _arq
from openstack.accelerator.v2 import attribute as _attribute
from openstack.accelerator.v2 import deployable as _deployable
from openstack.accelerator.v2 import device as _device
from openstack.accelerator.v2 import device_profile as _device_profile
from openstack import proxy
from openstack import resource


class Proxy(proxy.Proxy):
    # ========== Deployables ==========

    def deployables(self, **query):
        """Retrieve a generator of deployables.

        :param kwargs query: Optional query parameters to be sent to
            restrict the deployables to be returned.
        :returns: A generator of deployable instances.
        """
        return self._list(_deployable.Deployable, **query)

    def get_deployable(self, uuid, fields=None):
        """Get a single deployable.

        :param uuid: The value can be the UUID of a deployable.
        :returns: One :class:`~openstack.accelerator.v2.deployable.Deployable`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            deployable matching the criteria could be found.
        """
        return self._get(_deployable.Deployable, uuid)

    def update_deployable(self, uuid, patch):
        """Reconfig the FPGA with new bitstream.

        :param uuid: The value can be the UUID of a deployable
        :param patch: The information to reconfig.
        :returns: The results of FPGA reconfig.
        """
        return self._get_resource(_deployable.Deployable, uuid).patch(
            self, patch
        )

    # ========== Devices ==========

    def devices(self, **query):
        """Retrieve a generator of devices.

        :param kwargs query: Optional query parameters to be sent to
            restrict the devices to be returned. Available parameters include:

            * hostname: The hostname of the device.
            * type: The type of the device.
            * vendor: The vendor ID of the device.
            * sort: A list of sorting keys separated by commas. Each sorting
              key can optionally be attached with a sorting direction
              modifier which can be ``asc`` or ``desc``.
            * limit: Requests a specified size of returned items from the
              query. Returns a number of items up to the specified limit
              value.
            * marker: Specifies the ID of the last-seen item. Use the limit
              parameter to make an initial limited request and use the ID of
              the last-seen item from the response as the marker parameter
              value in a subsequent limited request.
        :returns: A generator of device instances.
        """
        return self._list(_device.Device, **query)

    def get_device(self, uuid, fields=None):
        """Get a single device.

        :param uuid: The value can be the UUID of a device.
        :returns: One :class:`~openstack.accelerator.v2.device.Device`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            device matching the criteria could be found.
        """
        return self._get(_device.Device, uuid)

    # ========== Device profiles ==========

    def device_profiles(self, **query):
        """Retrieve a generator of device profiles.

        :param kwargs query: Optional query parameters to be sent to
            restrict the device profiles to be returned.
        :returns: A generator of device profile instances.
        """
        return self._list(_device_profile.DeviceProfile, **query)

    def create_device_profile(self, **attrs):
        """Create a device_profile.

        :param kwargs attrs: a list of device_profiles.
        :returns: The list of created device profiles
        """
        return self._create(_device_profile.DeviceProfile, **attrs)

    def delete_device_profile(self, device_profile, ignore_missing=True):
        """Delete a device profile

        :param device_profile: The value can be either the ID of a device
            profile or a
            :class:`~openstack.accelerator.v2.device_profile.DeviceProfile`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the device profile does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent device profile.
        :returns: ``None``
        """
        return self._delete(
            _device_profile.DeviceProfile,
            device_profile,
            ignore_missing=ignore_missing,
        )

    def get_device_profile(self, uuid, fields=None):
        """Get a single device profile.

        :param uuid: The value can be the UUID of a device profile.
        :returns: One :class:
            `~openstack.accelerator.v2.device_profile.DeviceProfile`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            device profile matching the criteria could be found.
        """
        return self._get(_device_profile.DeviceProfile, uuid)

    # ========== Accelerator requests ==========

    def accelerator_requests(self, **query):
        """Retrieve a generator of accelerator requests.

        :param kwargs query: Optional query parameters to be sent to
            restrict the accelerator requests to be returned.
        :returns: A generator of accelerator request instances.
        """
        return self._list(_arq.AcceleratorRequest, **query)

    def create_accelerator_request(self, **attrs):
        """Create an ARQs for a single device profile.

        :param kwargs attrs: request body.
        :returns: The created accelerator request instance.
        """
        return self._create(_arq.AcceleratorRequest, **attrs)

    def delete_accelerator_request(
        self,
        accelerator_request,
        ignore_missing=True,
    ):
        """Delete a device profile

        :param device_profile: The value can be either the ID of a device
            profile or a
            :class:`~openstack.accelerator.v2.device_profile.DeviceProfile`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be raised
            when the device profile does not exist. When set to ``True``, no
            exception will be set when attempting to delete a nonexistent
            accelerator request.
        :returns: ``None``
        """
        return self._delete(
            _arq.AcceleratorRequest,
            accelerator_request,
            ignore_missing=ignore_missing,
        )

    def get_accelerator_request(self, uuid, fields=None):
        """Get a single accelerator request.

        :param uuid: The value can be the UUID of a accelerator request.
        :returns: One :class:
            `~openstack.accelerator.v2.accelerator_request.AcceleratorRequest`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            accelerator request matching the criteria could be found.
        """
        return self._get(_arq.AcceleratorRequest, uuid)

    def update_accelerator_request(self, uuid, properties):
        """Bind/Unbind an accelerator to VM.

        :param uuid: The uuid of the accelerator_request to be bound/unbound.
        :param properties: The info of VM
            that will bind/unbind the accelerator.
        :returns: True if bind/unbind succeeded, False otherwise.
        """
        return self._get_resource(_arq.AcceleratorRequest, uuid).patch(
            self, properties
        )

    # ========== Attributes ==========

    def attributes(self, **query):
        """Retrieve a generator of attributes.

        :param kwargs query: Optional query parameters to be sent to
            restrict the attributes to be returned.
        :returns: A generator of attribute instances.
        """
        return self._list(_attribute.Attribute, **query)

    def create_attribute(self, **attrs):
        """Create a attribute.

        :param kwargs attrs: a list of attributes.
        :returns: The list of created attributes
        """
        return self._create(_attribute.Attribute, **attrs)

    def delete_attribute(self, attribute, ignore_missing=True):
        """Delete a attribute

        :param attribute: The value can be either the ID of a attributes or a
            :class:`~openstack.accelerator.v2.attribute.Attributes`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the device profile does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent device profile.
        :returns: ``None``
        """
        return self._delete(
            _attribute.Attribute,
            attribute,
            ignore_missing=ignore_missing,
        )

    def get_attribute(self, uuid, fields=None):
        """Get a single device profile.

        :param uuid: The value can be the UUID of a attribute.
        :returns: One :class:
            `~openstack.accelerator.v2.attribute.Attribute`
        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
            device profile matching the criteria could be found.
        """
        return self._get(_attribute.Attribute, uuid)

    # ========== Utilities ==========

    def wait_for_status(
        self,
        res: resource.ResourceT,
        status: str,
        failures: list[str] | None = None,
        interval: int | float | None = 2,
        wait: int | None = None,
        attribute: str = 'status',
        callback: ty.Callable[[int], None] | None = None,
    ) -> resource.ResourceT:
        """Wait for the resource to be in a particular status.

        :param session: The session to use for making this request.
        :param resource: The resource to wait on to reach the status. The
            resource must have a status attribute specified via ``attribute``.
        :param status: Desired status of the resource.
        :param failures: Statuses that would indicate the transition
            failed such as 'ERROR'. Defaults to ['ERROR'].
        :param interval: Number of seconds to wait between checks.
        :param wait: Maximum number of seconds to wait for transition.
            Set to ``None`` to wait forever.
        :param attribute: Name of the resource attribute that contains the
            status.
        :param callback: A callback function. This will be called with a single
            value, progress. This is API specific but is generally a percentage
            value from 0-100.

        :return: The updated resource.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
            transition to status failed to occur in ``wait`` seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
            transitioned to one of the states in ``failures``.
        :raises: :class:`~AttributeError` if the resource does not have a
            ``status`` attribute
        """
        return resource.wait_for_status(
            self, res, status, failures, interval, wait, attribute, callback
        )

    def wait_for_delete(
        self,
        res: resource.ResourceT,
        interval: int = 2,
        wait: int = 120,
        callback: ty.Callable[[int], None] | None = None,
    ) -> resource.ResourceT:
        """Wait for a resource to be deleted.

        :param res: The resource to wait on to be deleted.
        :param interval: Number of seconds to wait before to consecutive
            checks.
        :param wait: Maximum number of seconds to wait before the change.
        :param callback: A callback function. This will be called with a single
            value, progress, which is a percentage value from 0-100.

        :returns: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
            to delete failed to occur in the specified seconds.
        """
        return resource.wait_for_delete(self, res, interval, wait, callback)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_project_cleanup
----------------------------------

Functional tests for project cleanup methods.
"""

import queue

from openstack.network.v2 import network as _network
from openstack import resource
from openstack.tests.functional import base


class TestProjectCleanup(base.BaseFunctionalTest):
    _wait_for_timeout_key = 'OPENSTACKSDK_FUNC_TEST_TIMEOUT_CLEANUP'

    def setUp(self):
        super().setUp()
        if not self.user_cloud_alt:
            self.skipTest("Alternate demo cloud is required for this test")

        self.network_name = self.getUniqueString('network')

    def _create_network_resources(self):
        self.net = self.user_cloud_alt.network.create_network(
            name=self.network_name,
        )
        self.subnet = self.user_cloud_alt.network.create_subnet(
            name=self.getUniqueString('subnet'),
            network_id=self.net.id,
            cidr='192.169.1.0/24',
            ip_version=4,
        )
        self.router = self.user_cloud_alt.network.create_router(
            name=self.getUniqueString('router')
        )
        self.user_cloud_alt.network.add_interface_to_router(
            self.router.id, subnet_id=self.subnet.id
        )

    def test_cleanup(self):
        self._create_network_resources()
        status_queue: queue.Queue[resource.Resource] = queue.Queue()

        # First round - check no resources are old enough
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'created_at': '2000-01-01'},
        )

        self.assertTrue(status_queue.empty())

        # Second round - resource evaluation function return false, ensure
        # nothing identified
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'created_at': '2200-01-01'},
            resource_evaluation_fn=lambda x, y, z: False,
        )

        self.assertTrue(status_queue.empty())

        # Third round - filters set too low
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'created_at': '2200-01-01'},
        )

        objects = []
        while not status_queue.empty():
            objects.append(status_queue.get())

        # At least known networks should be identified
        net_names = list(obj.name for obj in objects)
        self.assertIn(self.network_name, net_names)

        # Fourth round - dry run with no filters, ensure everything identified
        self.user_cloud_alt.project_cleanup(
            dry_run=True, wait_timeout=120, status_queue=status_queue
        )

        objects = []
        while not status_queue.empty():
            objects.append(status_queue.get())

        net_names = list(obj.name for obj in objects)
        self.assertIn(self.network_name, net_names)

        # Ensure network still exists
        net = self.user_cloud_alt.network.get_network(self.net.id)
        self.assertEqual(net.name, self.net.name)

        # Last round - do a real cleanup
        self.user_cloud_alt.project_cleanup(
            dry_run=False, wait_timeout=600, status_queue=status_queue
        )

        objects = []
        while not status_queue.empty():
            objects.append(status_queue.get())

        nets = self.user_cloud_alt.network.networks()
        net_names = list(obj.name for obj in nets)
        # Since we might not have enough privs to drop all nets - ensure
        # we do not have our known one
        self.assertNotIn(self.network_name, net_names)

    def test_block_storage_cleanup(self):
        if not self.user_cloud.has_service('object-store'):
            self.skipTest('Object service is requred, but not available')

        status_queue: queue.Queue[resource.Resource] = queue.Queue()

        vol = self.user_cloud_alt.block_storage.create_volume(
            name='vol1', size='1'
        )
        self.user_cloud_alt.block_storage.wait_for_status(vol)
        s1 = self.user_cloud_alt.block_storage.create_snapshot(
            volume_id=vol.id
        )
        self.user_cloud_alt.block_storage.wait_for_status(s1)
        b1 = self.user_cloud_alt.block_storage.create_backup(volume_id=vol.id)
        self.user_cloud_alt.block_storage.wait_for_status(b1)
        b2 = self.user_cloud_alt.block_storage.create_backup(
            volume_id=vol.id, is_incremental=True, snapshot_id=s1.id
        )
        self.user_cloud_alt.block_storage.wait_for_status(b2)
        b3 = self.user_cloud_alt.block_storage.create_backup(
            volume_id=vol.id, is_incremental=True, snapshot_id=s1.id
        )
        self.user_cloud_alt.block_storage.wait_for_status(b3)

        # First round - check no resources are old enough
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'created_at': '2000-01-01'},
        )

        self.assertTrue(status_queue.empty())

        # Second round - resource evaluation function return false, ensure
        # nothing identified
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'created_at': '2200-01-01'},
            resource_evaluation_fn=lambda x, y, z: False,
        )

        self.assertTrue(status_queue.empty())

        # Third round - filters set too low
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'created_at': '2200-01-01'},
        )

        objects = []
        while not status_queue.empty():
            objects.append(status_queue.get())

        # At least known networks should be identified
        volumes = list(obj.id for obj in objects)
        self.assertIn(vol.id, volumes)

        # Fourth round - dry run with no filters, ensure everything identified
        self.user_cloud_alt.project_cleanup(
            dry_run=True, wait_timeout=120, status_queue=status_queue
        )

        objects = []
        while not status_queue.empty():
            objects.append(status_queue.get())

        vol_ids = list(obj.id for obj in objects)
        self.assertIn(vol.id, vol_ids)

        # Ensure volume still exists
        vol_check = self.user_cloud_alt.block_storage.get_volume(vol.id)
        self.assertEqual(vol.name, vol_check.name)

        # Last round - do a real cleanup
        self.user_cloud_alt.project_cleanup(
            dry_run=False, wait_timeout=600, status_queue=status_queue
        )
        # Ensure no backups remain
        self.assertEqual(
            0, len(list(self.user_cloud_alt.block_storage.backups()))
        )
        # Ensure no snapshots remain
        self.assertEqual(
            0, len(list(self.user_cloud_alt.block_storage.snapshots()))
        )

    def test_cleanup_swift(self):
        if not self.user_cloud.has_service('object-store'):
            self.skipTest('Object service is requred, but not available')

        status_queue: queue.Queue[resource.Resource] = queue.Queue()

        self.user_cloud_alt.object_store.create_container('test_cleanup')
        for i in range(1, 10):
            self.user_cloud_alt.object_store.create_object(
                "test_cleanup", f"test{i}", data="test{i}"
            )

        # First round - check no resources are old enough
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'updated_at': '2000-01-01'},
        )

        self.assertTrue(status_queue.empty())

        # Second round - filters set too low
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'updated_at': '2200-01-01'},
        )
        objects = []
        while not status_queue.empty():
            objects.append(status_queue.get())

        # At least known objects should be identified
        obj_names = list(obj.name for obj in objects)
        self.assertIn('test1', obj_names)

        # Ensure object still exists
        obj = self.user_cloud_alt.object_store.get_object(
            "test1", "test_cleanup"
        )
        self.assertIsNotNone(obj)

        # Last round - do a real cleanup
        self.user_cloud_alt.project_cleanup(
            dry_run=False, wait_timeout=600, status_queue=status_queue
        )

        objects.clear()
        while not status_queue.empty():
            objects.append(status_queue.get())
        self.assertIsNone(self.user_cloud_alt.get_container('test_container'))

    def test_cleanup_vpnaas(self):
        if not list(
            self.user_cloud_alt.network.service_providers(service_type="VPN")
        ):
            self.skipTest("VPNaaS plugin is requred, but not available")

        status_queue: queue.Queue[resource.Resource] = queue.Queue()

        # Find available external networks and use one
        for network in self.user_cloud_alt.network.networks():
            if network.is_router_external:
                external_network: _network.Network = network
                break
        else:
            self.skipTest("External network is required, but not available")

        # Create left network resources
        network_left = self.user_cloud_alt.network.create_network(
            name="network_left"
        )
        subnet_left = self.user_cloud_alt.network.create_subnet(
            name="subnet_left",
            network_id=network_left.id,
            cidr="192.168.1.0/24",
            ip_version=4,
        )
        router_left = self.user_cloud_alt.network.create_router(
            name="router_left"
        )
        self.user_cloud_alt.network.add_interface_to_router(
            router=router_left.id, subnet_id=subnet_left.id
        )
        router_left = self.user_cloud_alt.network.update_router(
            router_left,
            external_gateway_info={"network_id": external_network.id},
        )

        # Create right network resources
        network_right = self.user_cloud_alt.network.create_network(
            name="network_right"
        )
        subnet_right = self.user_cloud_alt.network.create_subnet(
            name="subnet_right",
            network_id=network_right.id,
            cidr="192.168.2.0/24",
            ip_version=4,
        )
        router_right = self.user_cloud_alt.network.create_router(
            name="router_right"
        )
        self.user_cloud_alt.network.add_interface_to_router(
            router=router_right.id, subnet_id=subnet_right.id
        )
        router_right = self.user_cloud_alt.network.update_router(
            router_right,
            external_gateway_info={"network_id": external_network.id},
        )

        # Create VPNaaS resources
        ike_policy = self.user_cloud_alt.network.create_vpn_ike_policy(
            name="ike_policy"
        )
        ipsec_policy = self.user_cloud_alt.network.create_vpn_ipsec_policy(
            name="ipsec_policy"
        )

        vpn_service = self.user_cloud_alt.network.create_vpn_service(
            name="vpn_service", router_id=router_left.id
        )

        ep_group_local = self.user_cloud_alt.network.create_vpn_endpoint_group(
            name="endpoint_group_local",
            type="subnet",
            endpoints=[subnet_left.id],
        )
        ep_group_peer = self.user_cloud_alt.network.create_vpn_endpoint_group(
            name="endpoint_group_peer",
            type="cidr",
            endpoints=[subnet_right.cidr],
        )

        router_right_ip = router_right.external_gateway_info[
            'external_fixed_ips'
        ][0]['ip_address']
        ipsec_site_conn = (
            self.user_cloud_alt.network.create_vpn_ipsec_site_connection(
                name="ipsec_site_connection",
                vpnservice_id=vpn_service.id,
                ikepolicy_id=ike_policy.id,
                ipsecpolicy_id=ipsec_policy.id,
                local_ep_group_id=ep_group_local.id,
                peer_ep_group_id=ep_group_peer.id,
                psk="test",
                peer_address=router_right_ip,
                peer_id=router_right_ip,
            )
        )

        # First round - check no resources are old enough
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'created_at': '2000-01-01'},
        )
        self.assertTrue(status_queue.empty())

        # Second round - resource evaluation function return false, ensure
        # nothing identified
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'created_at': '2200-01-01'},
            resource_evaluation_fn=lambda x, y, z: False,
        )
        self.assertTrue(status_queue.empty())

        # Third round - filters set too low
        self.user_cloud_alt.project_cleanup(
            dry_run=True,
            wait_timeout=120,
            status_queue=status_queue,
            filters={'created_at': '2200-01-01'},
        )
        objects = []
        while not status_queue.empty():
            objects.append(status_queue.get())

        # VPN resources do not have a created_at property
        # Check for the network instead
        resource_ids = list(obj.id for obj in objects)
        self.assertIn(network_left.id, resource_ids)

        # Fourth round - dry run with no filters, ensure everything identified
        self.user_cloud_alt.project_cleanup(
            dry_run=True, wait_timeout=120, status_queue=status_queue
        )
        objects = []
        while not status_queue.empty():
            objects.append(status_queue.get())

        resource_ids = list(obj.id for obj in objects)
        self.assertIn(ipsec_site_conn.id, resource_ids)

        # Ensure vpn resources still exist
        site_conn_check = (
            self.user_cloud_alt.network.get_vpn_ipsec_site_connection(
                ipsec_site_conn.id
            )
        )
        self.assertEqual(site_conn_check.name, ipsec_site_conn.name)

        # Last round - do a real cleanup
        self.user_cloud_alt.project_cleanup(
            dry_run=False, wait_timeout=600, status_queue=status_queue
        )
        # Ensure no VPN resources remain
        self.assertEqual(
            0, len(list(self.user_cloud_alt.network.vpn_ike_policies()))
        )
        self.assertEqual(
            0, len(list(self.user_cloud_alt.network.vpn_ipsec_policies()))
        )
        self.assertEqual(
            0, len(list(self.user_cloud_alt.network.vpn_services()))
        )
        self.assertEqual(
            0, len(list(self.user_cloud_alt.network.vpn_endpoint_groups()))
        )
        self.assertEqual(
            0,
            len(
                list(self.user_cloud_alt.network.vpn_ipsec_site_connections())
            ),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.functional import base


class TestQuota(base.BaseFunctionalTest):
    def setUp(self):
        super().setUp()

        self.require_service("dns")
        if not self._operator_cloud_name:
            self.skip("Operator cloud must be set for this test")

        self.project = self.create_temporary_project()

    def test_quota(self):
        # set quota

        attrs = {
            "api_export_size": 1,
            "recordset_records": 2,
            "zone_records": 3,
            "zone_recordsets": 4,
            "zones": 5,
        }
        new_quota = self.operator_cloud.dns.update_quota(
            self.project.id, **attrs
        )
        self.assertEqual(attrs["api_export_size"], new_quota.api_export_size)
        self.assertEqual(
            attrs["recordset_records"], new_quota.recordset_records
        )
        self.assertEqual(attrs["zone_records"], new_quota.zone_records)
        self.assertEqual(attrs["zone_recordsets"], new_quota.zone_recordsets)
        self.assertEqual(attrs["zones"], new_quota.zones)

        # get quota

        expected_keys = [
            "id",
            "api_export_size",
            "recordset_records",
            "zone_records",
            "zone_recordsets",
            "zones",
        ]
        test_quota = self.operator_cloud.dns.get_quota(self.project.id)
        for actual_key in test_quota._body.attributes.keys():
            self.assertIn(actual_key, expected_keys)
        self.assertEqual(self.project.id, test_quota.id)
        self.assertEqual(attrs["api_export_size"], test_quota.api_export_size)
        self.assertEqual(
            attrs["recordset_records"], test_quota.recordset_records
        )
        self.assertEqual(attrs["zone_records"], test_quota.zone_records)
        self.assertEqual(attrs["zone_recordsets"], test_quota.zone_recordsets)
        self.assertEqual(attrs["zones"], test_quota.zones)

        # reset quota

        self.operator_cloud.dns.delete_quota(self.project.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import typing as ty

from openstack.clustering.v1 import action as _action
from openstack.clustering.v1 import build_info
from openstack.clustering.v1 import cluster as _cluster
from openstack.clustering.v1 import cluster_attr as _cluster_attr
from openstack.clustering.v1 import cluster_policy as _cluster_policy
from openstack.clustering.v1 import event as _event
from openstack.clustering.v1 import node as _node
from openstack.clustering.v1 import policy as _policy
from openstack.clustering.v1 import policy_type as _policy_type
from openstack.clustering.v1 import profile as _profile
from openstack.clustering.v1 import profile_type as _profile_type
from openstack.clustering.v1 import receiver as _receiver
from openstack.clustering.v1 import service as _service
from openstack import proxy
from openstack import resource


class Proxy(proxy.Proxy):
    _resource_registry = {
        "action": _action.Action,
        "build_info": build_info.BuildInfo,
        "cluster": _cluster.Cluster,
        "cluster_attr": _cluster_attr.ClusterAttr,
        "cluster_policy": _cluster_policy.ClusterPolicy,
        "event": _event.Event,
        "node": _node.Node,
        "policy": _policy.Policy,
        "policy_type": _policy_type.PolicyType,
        "profile": _profile.Profile,
        "profile_type": _profile_type.ProfileType,
        "receiver": _receiver.Receiver,
        "service": _service.Service,
    }

    def get_build_info(self):
        """Get build info for service engine and API

        :returns: A dictionary containing the API and engine revision string.
        """
        return self._get(build_info.BuildInfo, requires_id=False)

    def profile_types(self, **query):
        """Get a generator of profile types.

        :returns: A generator of objects that are of type
            :class:`~openstack.clustering.v1.profile_type.ProfileType`
        """
        return self._list(_profile_type.ProfileType, **query)

    def get_profile_type(self, profile_type):
        """Get the details about a profile type.

        :param profile_type: The name of the profile_type to retrieve or an
            object of
            :class:`~openstack.clustering.v1.profile_type.ProfileType`.

        :returns: A :class:`~openstack.clustering.v1.profile_type.ProfileType`
            object.
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            profile_type matching the name could be found.
        """
        return self._get(_profile_type.ProfileType, profile_type)

    def policy_types(self, **query):
        """Get a generator of policy types.

        :returns: A generator of objects that are of type
            :class:`~openstack.clustering.v1.policy_type.PolicyType`
        """
        return self._list(_policy_type.PolicyType, **query)

    def get_policy_type(self, policy_type):
        """Get the details about a policy type.

        :param policy_type: The name of a poicy_type or an object of
            :class:`~openstack.clustering.v1.policy_type.PolicyType`.

        :returns: A :class:`~openstack.clustering.v1.policy_type.PolicyType`
            object.
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            policy_type matching the name could be found.
        """
        return self._get(_policy_type.PolicyType, policy_type)

    def create_profile(self, **attrs):
        """Create a new profile from attributes.

        :param dict attrs: Keyword arguments that will be used to create a
            :class:`~openstack.clustering.v1.profile.Profile`, it is comprised
            of the properties on the Profile class.

        :returns: The results of profile creation.
        :rtype: :class:`~openstack.clustering.v1.profile.Profile`.
        """
        return self._create(_profile.Profile, **attrs)

    def delete_profile(self, profile, ignore_missing=True):
        """Delete a profile.

        :param profile: The value can be either the name or ID of a profile or
            a :class:`~openstack.clustering.v1.profile.Profile` instance.
        :param bool ignore_missing: When set to ``False``, an exception
            :class:`~openstack.exceptions.NotFoundException` will be raised
            when the profile could not be found. When set to ``True``, no
            exception will be raised when attempting to delete a non-existent
            profile.

        :returns: ``None``
        """
        self._delete(_profile.Profile, profile, ignore_missing=ignore_missing)

    def find_profile(self, name_or_id, ignore_missing=True):
        """Find a single profile.

        :param str name_or_id: The name or ID of a profile.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.clustering.v1.profile.Profile` object
            or None
        """
        return self._find(
            _profile.Profile, name_or_id, ignore_missing=ignore_missing
        )

    def get_profile(self, profile):
        """Get a single profile.

        :param profile: The value can be the name or ID of a profile or a
            :class:`~openstack.clustering.v1.profile.Profile` instance.

        :returns: One :class:`~openstack.clustering.v1.profile.Profile`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            profile matching the criteria could be found.
        """
        return self._get(_profile.Profile, profile)

    def profiles(self, **query):
        """Retrieve a generator of profiles.

        :param kwargs query: Optional query parameters to be sent to
            restrict the profiles to be returned. Available parameters include:

            * name: The name of a profile.
            * type: The type name of a profile.
            * metadata: A list of key-value pairs that are associated with a
              profile.
            * sort: A list of sorting keys separated by commas. Each sorting
              key can optionally be attached with a sorting direction
              modifier which can be ``asc`` or ``desc``.
            * limit: Requests a specified size of returned items from the
              query.  Returns a number of items up to the specified limit
              value.
            * marker: Specifies the ID of the last-seen item. Use the limit
              parameter to make an initial limited request and use the ID of
              the last-seen item from the response as the marker parameter
              value in a subsequent limited request.
            * global_project: A boolean value indicating whether profiles
              from all projects will be returned.

        :returns: A generator of profile instances.
        """
        return self._list(_profile.Profile, **query)

    def update_profile(self, profile, **attrs):
        """Update a profile.

        :param profile: Either the name or the ID of the profile, or an
            instance of :class:`~openstack.clustering.v1.profile.Profile`.
        :param attrs: The attributes to update on the profile represented by
            the ``value`` parameter.

        :returns: The updated profile.
        :rtype: :class:`~openstack.clustering.v1.profile.Profile`
        """
        return self._update(_profile.Profile, profile, **attrs)

    def validate_profile(self, **attrs):
        """Validate a profile spec.

        :param dict attrs: Keyword arguments that will be used to create a
            :class:`~openstack.clustering.v1.profile.ProfileValidate`, it is
            comprised of the properties on the Profile class.

        :returns: The results of profile validation.
        :rtype: :class:`~openstack.clustering.v1.profile.ProfileValidate`.
        """
        return self._create(_profile.ProfileValidate, **attrs)

    # ====== CLUSTERS ======
    def create_cluster(self, **attrs):
        """Create a new cluster from attributes.

        :param dict attrs: Keyword arguments that will be used to create a
            :class:`~openstack.clustering.v1.cluster.Cluster`, it is comprised
            of the properties on the Cluster class.

        :returns: The results of cluster creation.
        :rtype: :class:`~openstack.clustering.v1.cluster.Cluster`.
        """
        return self._create(_cluster.Cluster, **attrs)

    def delete_cluster(self, cluster, ignore_missing=True, force_delete=False):
        """Delete a cluster.

        :param cluster: The value can be either the name or ID of a cluster or
            a :class:`~openstack.cluster.v1.cluster.Cluster` instance.
        :param bool ignore_missing: When set to ``False``, an exception
            :class:`~openstack.exceptions.NotFoundException` will be raised
            when the cluster could not be found. When set to ``True``, no
            exception will be raised when attempting to delete a non-existent
            cluster.
        :param bool force_delete: When set to ``True``, the cluster deletion
            will be forced immediately.

        :returns: The instance of the Cluster which was deleted.
        :rtype: :class:`~openstack.cluster.v1.cluster.Cluster`.
        """
        if force_delete:
            server = self._get_resource(_cluster.Cluster, cluster)
            return server.force_delete(self)
        else:
            return self._delete(
                _cluster.Cluster, cluster, ignore_missing=ignore_missing
            )

    def find_cluster(self, name_or_id, ignore_missing=True):
        """Find a single cluster.

        :param str name_or_id: The name or ID of a cluster.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~openstack.clustering.v1.cluster.Cluster` object
            or None
        """
        return self._find(
            _cluster.Cluster, name_or_id, ignore_missing=ignore_missing
        )

    def get_cluster(self, cluster):
        """Get a single cluster.

        :param cluster: The value can be the name or ID of a cluster or a
            :class:`~openstack.clustering.v1.cluster.Cluster` instance.

        :returns: One :class:`~openstack.clustering.v1.cluster.Cluster`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            cluster matching the criteria could be found.
        """
        return self._get(_cluster.Cluster, cluster)

    def clusters(self, **query):
        """Retrieve a generator of clusters.

        :param kwargs query: Optional query parameters to be sent to
            restrict the clusters to be returned. Available parameters include:

            * name: The name of a cluster.
            * status: The current status of a cluster.
            * sort: A list of sorting keys separated by commas. Each sorting
              key can optionally be attached with a sorting direction
              modifier which can be ``asc`` or ``desc``.
            * limit: Requests a specified size of returned items from the
              query.  Returns a number of items up to the specified limit
              value.
            * marker: Specifies the ID of the last-seen item. Use the limit
              parameter to make an initial limited request and use the ID of
              the last-seen item from the response as the marker parameter
              value in a subsequent limited request.
            * global_project: A boolean value indicating whether clusters
              from all projects will be returned.

        :returns: A generator of cluster instances.
        """
        return self._list(_cluster.Cluster, **query)

    def update_cluster(self, cluster, **attrs):
        """Update a cluster.

        :param cluster: Either the name or the ID of the cluster, or an
            instance of :class:`~openstack.clustering.v1.cluster.Cluster`.
        :param attrs: The attributes to update on the cluster represented by
            the ``cluster`` parameter.

        :returns: The updated cluster.
        :rtype: :class:`~openstack.clustering.v1.cluster.Cluster`
        """
        return self._update(_cluster.Cluster, cluster, **attrs)

    def get_cluster_metadata(self, cluster):
        """Return a dictionary of metadata for a cluster

        :param cluster: Either the ID of a cluster or a
            :class:`~openstack.clustering.v3.cluster.Cluster`.

        :returns: A :class:`~openstack.clustering.v3.cluster.Cluster` with the
            cluster's metadata. All keys and values are Unicode text.
        :rtype: :class:`~openstack.clustering.v3.cluster.Cluster`
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.fetch_metadata(self)

    def set_cluster_metadata(self, cluster, **metadata):
        """Update metadata for a cluster

        :param cluster: Either the ID of a cluster or a
            :class:`~openstack.clustering.v3.cluster.Cluster`.
        :param kwargs metadata: Key/value pairs to be updated in the cluster's
            metadata. No other metadata is modified by this call. All keys
            and values are stored as Unicode.


        :returns: A :class:`~openstack.clustering.v3.cluster.Cluster` with the
            cluster's metadata. All keys and values are Unicode text.
        :rtype: :class:`~openstack.clustering.v3.cluster.Cluster`
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.set_metadata(self, metadata=metadata)

    def delete_cluster_metadata(self, cluster, keys=None):
        """Delete metadata for a cluster

        :param cluster: Either the ID of a cluster or a
            :class:`~openstack.clustering.v3.cluster.Cluster`.
        :param list keys: The keys to delete. If left empty complete
            metadata will be removed.

        :rtype: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        if keys is not None:
            for key in keys:
                cluster.delete_metadata_item(self, key)
        else:
            cluster.delete_metadata(self)

    def add_nodes_to_cluster(self, cluster, nodes):
        """Add nodes to a cluster.

        :param cluster: Either the name or the ID of the cluster, or an
            instance of :class:`~openstack.clustering.v1.cluster.Cluster`.
        :param nodes: List of nodes to be added to the cluster.
        :returns: A dict containing the action initiated by this operation.
        """
        if isinstance(cluster, _cluster.Cluster):
            obj = cluster
        else:
            obj = self._find(_cluster.Cluster, cluster, ignore_missing=False)
        return obj.add_nodes(self, nodes)

    def remove_nodes_from_cluster(self, cluster, nodes, **params):
        """Remove nodes from a cluster.

        :param cluster: Either the name or the ID of the cluster, or an
            instance of :class:`~openstack.clustering.v1.cluster.Cluster`.
        :param nodes: List of nodes to be removed from the cluster.
        :param kwargs params: Optional query parameters to be sent to
            restrict the nodes to be returned. Available parameters include:

            * destroy_after_deletion: A boolean value indicating whether the
              deleted nodes to be destroyed right away.
        :returns: A dict containing the action initiated by this operation.
        """
        if isinstance(cluster, _cluster.Cluster):
            obj = cluster
        else:
            obj = self._find(_cluster.Cluster, cluster, ignore_missing=False)
        return obj.del_nodes(self, nodes, **params)

    def replace_nodes_in_cluster(self, cluster, nodes):
        """Replace the nodes in a cluster with specified nodes.

        :param cluster: Either the name or the ID of the cluster, or an
            instance of :class:`~openstack.clustering.v1.cluster.Cluster`.
        :param nodes: List of nodes to be deleted/added to the cluster.
        :returns: A dict containing the action initiated by this operation.
        """
        if isinstance(cluster, _cluster.Cluster):
            obj = cluster
        else:
            obj = self._find(_cluster.Cluster, cluster, ignore_missing=False)
        return obj.replace_nodes(self, nodes)

    def scale_out_cluster(self, cluster, count=None):
        """Inflate the size of a cluster.

        :param cluster: Either the name or the ID of the cluster, or an
            instance of :class:`~openstack.clustering.v1.cluster.Cluster`.
        :param count: Optional parameter specifying the number of nodes to
            be added.
        :returns: A dict containing the action initiated by this operation.
        """
        if isinstance(cluster, _cluster.Cluster):
            obj = cluster
        else:
            obj = self._find(_cluster.Cluster, cluster, ignore_missing=False)
        return obj.scale_out(self, count)

    def scale_in_cluster(self, cluster, count=None):
        """Shrink the size of a cluster.

        :param cluster: Either the name or the ID of the cluster, or an
            instance of :class:`~openstack.clustering.v1.cluster.Cluster`.
        :param count: Optional parameter specifying the number of nodes to
            be removed.
        :returns: A dict containing the action initiated by this operation.
        """
        if isinstance(cluster, _cluster.Cluster):
            obj = cluster
        else:
            obj = self._find(_cluster.Cluster, cluster, ignore_missing=False)
        return obj.scale_in(self, count)

    def resize_cluster(self, cluster, **params):
        """Resize of cluster.

        :param cluster: Either the name or the ID of the cluster, or an
            instance of :class:`~openstack.clustering.v1.cluster.Cluster`.
        :param dict params: A dictionary providing the parameters for the
            resize action.
        :returns: A dict containing the action initiated by this operation.
        """
        if isinstance(cluster, _cluster.Cluster):
            obj = cluster
        else:
            obj = self._find(_cluster.Cluster, cluster, ignore_missing=False)
        return obj.resize(self, **params)

    def attach_policy_to_cluster(self, cluster, policy, **params):
        """Attach a policy to a cluster.

        :param cluster: Either the name or the ID of the cluster, or an
            instance of :class:`~openstack.clustering.v1.cluster.Cluster`.
        :param policy: Either the name or the ID of a policy.
        :param dict params: A dictionary containing the properties for the
            policy to be attached.
        :returns: A dict containing the action initiated by this operation.
        """
        if isinstance(cluster, _cluster.Cluster):
            obj = cluster
        else:
            obj = self._find(_cluster.Cluster, cluster, ignore_missing=False)
        return obj.policy_attach(self, policy, **params)

    def detach_policy_from_cluster(self, cluster, policy):
        """Detach a policy from a cluster.

        :param cluster: Either the name or the ID of the cluster, or an
            instance of :class:`~openstack.clustering.v1.cluster.Cluster`.
        :param policy: Either the name or the ID of a policy.
        :returns: A dict containing the action initiated by this operation.
        """
        if isinstance(cluster, _cluster.Cluster):
            obj = cluster
        else:
            obj = self._find(_cluster.Cluster, cluster, ignore_missing=False)
        return obj.policy_detach(self, policy)

    def update_cluster_policy(self, cluster, policy, **params):
        """Change properties of a policy which is bound to the cluster.

        :param cluster: Either the name or the ID of the cluster, or an
            instance of :class:`~openstack.clustering.v1.cluster.Cluster`.
        :param policy: Either the name or the ID of a policy.
        :param dict params: A dictionary containing the new properties for
            the policy.
        :returns: A dict containing the action initiated by this operation.
        """
        if isinstance(cluster, _cluster.Cluster):
            obj = cluster
        else:
            obj = self._find(_cluster.Cluster, cluster, ignore_missing=False)
        return obj.policy_update(self, policy, **params)

    def collect_cluster_attrs(self, cluster, path, **query):
        """Collect attribute values across a cluster.

        :param cluster: The value can be either the ID of a cluster or a
            :class:`~openstack.clustering.v1.cluster.Cluster` instance.
        :param path: A Json path string specifying the attribute to collect.
        :param query: Optional query parameters to be sent to limit the
            resources being returned.

        :returns: A dictionary containing the list of attribute values.
        """
        return self._list(
            _cluster_attr.ClusterAttr, cluster_id=cluster, path=path
        )

    def check_cluster(self, cluster, **params):
        """Check a cluster.

        :param cluster: The value can be either the ID of a cluster or a
            :class:`~openstack.clustering.v1.cluster.Cluster` instance.
        :param dict params: A dictionary providing the parameters for the
            check action.

        :returns: A dictionary containing the action ID.
        """
        obj = self._get_resource(_cluster.Cluster, cluster)
        return obj.check(self, **params)

    def recover_cluster(self, cluster, **params):
        """recover a cluster.

        :param cluster: The value can be either the ID of a cluster or a
            :class:`~openstack.clustering.v1.cluster.Cluster` instance.
        :param dict params: A dictionary providing the parameters for the
            recover action.

        :returns: A dictionary containing the action ID.
        """
        obj = self._get_resource(_cluster.Cluster, cluster)
        return obj.recover(self, **params)

    def perform_operation_on_cluster(self, cluster, operation, **params):
        """Perform an operation on the specified cluster.

        :param cluster: The value can be either the ID of a cluster or a
            :class:`~openstack.clustering.v1.cluster.Cluster` instance.
        :param operation: A string specifying the operation to be performed.
        :param dict params: A dictionary providing the parameters for the
            operation.

        :returns: A dictionary containing the action ID.
        """
        obj = self._get_resource(_cluster.Cluster, cluster)
        return obj.op(self, operation, **params)

    def create_node(self, **attrs):
        """Create a new node from attributes.

        :param dict attrs: Keyword arguments that will be used to create a
            :class:`~openstack.clustering.v1.node.Node`, it is comprised
            of the properties on the ``Node`` class.

        :returns: The results of node creation.
        :rtype: :class:`~openstack.clustering.v1.node.Node`.
        """
        return self._create(_node.Node, **attrs)

    def delete_node(self, node, ignore_missing=True, force_delete=False):
        """Delete a node.

        :param node: The value can be either the name or ID of a node or a
            :class:`~openstack.cluster.v1.node.Node` instance.
        :param bool ignore_missing: When set to ``False``, an exception
            :class:`~openstack.exceptions.NotFoundException` will be raised
            when the node could not be found. When set to ``True``, no
            exception will be raised when attempting to delete a non-existent
            node.
        :param bool force_delete: When set to ``True``, the node deletion
            will be forced immediately.

        :returns: The instance of the Node which was deleted.
        :rtype: :class:`~openstack.cluster.v1.node.Node`.
        """
        if force_delete:
            server = self._get_resource(_node.Node, node)
            return server.force_delete(self)
        else:
            return self._delete(
                _node.Node, node, ignore_missing=ignore_missing
            )

    def find_node(self, name_or_id, ignore_missing=True):
        """Find a single node.

        :param str name_or_id: The name or ID of a node.
        :param bool ignore_missing: When set to "False"
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the specified node does not exist.
            when set to "True", None will be returned when
            attempting to find a nonexistent policy
        :returns: One :class:`~openstack.clustering.v1.node.Node` object
            or None.
        """
        return self._find(
            _node.Node, name_or_id, ignore_missing=ignore_missing
        )

    def get_node(self, node, details=False):
        """Get a single node.

        :param node: The value can be the name or ID of a node or a
            :class:`~openstack.clustering.v1.node.Node` instance.
        :param details: An optional argument that indicates whether the
            server should return more details when retrieving the node data.

        :returns: One :class:`~openstack.clustering.v1.node.Node`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            node matching the name or ID could be found.
        """
        # NOTE: When retrieving node with details (using NodeDetail resource),
        # the `node_id` is treated as part of the base_path thus a URI
        # property rather than a resource ID as assumed by the _get() method
        # in base proxy.
        if details:
            return self._get(_node.NodeDetail, requires_id=False, node_id=node)
        return self._get(_node.Node, node)

    def nodes(self, **query):
        """Retrieve a generator of nodes.

        :param kwargs query: Optional query parameters to be sent to
            restrict the nodes to be returned. Available parameters include:

            * cluster_id: A string including the name or ID of a cluster to
              which the resulted node(s) is a member.
            * name: The name of a node.
            * status: The current status of a node.
            * sort: A list of sorting keys separated by commas. Each sorting
              key can optionally be attached with a sorting direction
              modifier which can be ``asc`` or ``desc``.
            * limit: Requests at most the specified number of items be
              returned from the query.
            * marker: Specifies the ID of the last-seen node. Use the limit
              parameter to make an initial limited request and use the ID of
              the last-seen node from the response as the marker parameter
              value in a subsequent limited request.
            * global_project: A boolean value indicating whether nodes
              from all projects will be returned.

        :returns: A generator of node instances.
        """
        return self._list(_node.Node, **query)

    def update_node(self, node, **attrs):
        """Update a node.

        :param node: Either the name or the ID of the node, or an instance
            of :class:`~openstack.clustering.v1.node.Node`.
        :param attrs: The attributes to update on the node represented by
            the ``node`` parameter.

        :returns: The updated node.
        :rtype: :class:`~openstack.clustering.v1.node.Node`
        """
        return self._update(_node.Node, node, **attrs)

    def check_node(self, node, **params):
        """Check the health of the specified node.

        :param node: The value can be either the ID of a node or a
            :class:`~openstack.clustering.v1.node.Node` instance.
        :param dict params: A dictionary providing the parametes to the check
            action.

        :returns: A dictionary containing the action ID.
        """
        obj = self._get_resource(_node.Node, node)
        return obj.check(self, **params)

    def recover_node(self, node, **params):
        """Recover the specified node into healthy status.

        :param node: The value can be either the ID of a node or a
            :class:`~openstack.clustering.v1.node.Node` instance.
        :param dict params: A dict supplying parameters to the recover action.

        :returns: A dictionary containing the action ID.
        """
        obj = self._get_resource(_node.Node, node)
        return obj.recover(self, **params)

    def adopt_node(self, preview=False, **attrs):
        """Adopting an existing resource as a node.

        :param preview: A boolean indicating whether this is a "preview"
            operation which means only the profile to be used is returned
            rather than creating a node object using that profile.
        :param dict attrs: Keyword parameters for node adoption. Valid
            parameters include:

            * type: (Required) A string containing the profile type and
              version to be used for node adoption. For example,
              ``os.nova.sever-1.0``.
            * identity: (Required) A string including the name or ID of an
              OpenStack resource to be adopted as a Senlin node.
            * name: (Optional) The name of node to be created. Omitting
              this parameter will have the node named automatically.
            * snapshot: (Optional) A boolean indicating whether a snapshot
              of the target resource should be created if possible. Default
              is False.
            * metadata: (Optional) A dictionary of arbitrary key-value pairs
              to be associated with the adopted node.
            * overrides: (Optional) A dictionary of key-value pairs to be used
              to override attributes derived from the target resource.

        :returns: The result of node adoption. If `preview` is set to False
            (default), returns a :class:`~openstack.clustering.v1.node.Node`
            object, otherwise a Dict is returned containing the profile to
            be used for the new node.
        """
        node = self._get_resource(_node.Node, None)
        return node.adopt(self, preview=preview, **attrs)

    def perform_operation_on_node(self, node, operation, **params):
        """Perform an operation on the specified node.

        :param node: The value can be either the ID of a node or a
            :class:`~openstack.clustering.v1.node.Node` instance.
        :param operation: A string specifying the operation to be performed.
        :param dict params: A dictionary providing the parameters for the
            operation.

        :returns: A dictionary containing the action ID.
        """
        obj = self._get_resource(_node.Node, node)
        return obj.op(self, operation, **params)

    def create_policy(self, **attrs):
        """Create a new policy from attributes.

        :param dict attrs: Keyword arguments that will be used to create a
            :class:`~openstack.clustering.v1.policy.Policy`, it is comprised
            of the properties on the ``Policy`` class.

        :returns: The results of policy creation.
        :rtype: :class:`~openstack.clustering.v1.policy.Policy`.
        """
        return self._create(_policy.Policy, **attrs)

    def delete_policy(self, policy, ignore_missing=True):
        """Delete a policy.

        :param policy: The value can be either the name or ID of a policy or a
            :class:`~openstack.clustering.v1.policy.Policy` instance.
        :param bool ignore_missing: When set to ``False``, an exception
            :class:`~openstack.exceptions.NotFoundException` will be raised
            when the policy could not be found. When set to ``True``, no
            exception will be raised when attempting to delete a non-existent
            policy.

        :returns: ``None``
        """
        self._delete(_policy.Policy, policy, ignore_missing=ignore_missing)

    def find_policy(self, name_or_id, ignore_missing=True):
        """Find a single policy.

        :param str name_or_id: The name or ID of a policy.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the specified policy does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent policy.
        :returns: A policy object or None.
        :rtype: :class:`~openstack.clustering.v1.policy.Policy`
        """
        return self._find(
            _policy.Policy, name_or_id, ignore_missing=ignore_missing
        )

    def get_policy(self, policy):
        """Get a single policy.

        :param policy: The value can be the name or ID of a policy or a
            :class:`~openstack.clustering.v1.policy.Policy` instance.

        :returns: A policy object.
        :rtype: :class:`~openstack.clustering.v1.policy.Policy`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            policy matching the criteria could be found.
        """
        return self._get(_policy.Policy, policy)

    def policies(self, **query):
        """Retrieve a generator of policies.

        :param kwargs query: Optional query parameters to be sent to
            restrict the policies to be returned. Available parameters include:

            * name: The name of a policy.
            * type: The type name of a policy.
            * sort: A list of sorting keys separated by commas. Each sorting
              key can optionally be attached with a sorting direction
              modifier which can be ``asc`` or ``desc``.
            * limit: Requests a specified size of returned items from the
              query.  Returns a number of items up to the specified limit
              value.
            * marker: Specifies the ID of the last-seen item. Use the limit
              parameter to make an initial limited request and use the ID of
              the last-seen item from the response as the marker parameter
              value in a subsequent limited request.
            * global_project: A boolean value indicating whether policies from
              all projects will be returned.

        :returns: A generator of policy instances.
        """
        return self._list(_policy.Policy, **query)

    def update_policy(self, policy, **attrs):
        """Update a policy.

        :param policy: Either the name or the ID of a policy, or an instance
            of :class:`~openstack.clustering.v1.policy.Policy`.
        :param attrs: The attributes to update on the policy represented by
            the ``value`` parameter.

        :returns: The updated policy.
        :rtype: :class:`~openstack.clustering.v1.policy.Policy`
        """
        return self._update(_policy.Policy, policy, **attrs)

    def validate_policy(self, **attrs):
        """Validate a policy spec.

        :param dict attrs: Keyword arguments that will be used to create a
            :class:`~openstack.clustering.v1.policy.PolicyValidate`, it is
            comprised of the properties on the Policy class.

        :returns: The results of Policy validation.
        :rtype: :class:`~openstack.clustering.v1.policy.PolicyValidate`.
        """
        return self._create(_policy.PolicyValidate, **attrs)

    def cluster_policies(self, cluster, **query):
        """Retrieve a generator of cluster-policy bindings.

        :param cluster: The value can be the name or ID of a cluster or a
            :class:`~openstack.clustering.v1.cluster.Cluster` instance.
        :param kwargs query: Optional query parameters to be sent to
            restrict the policies to be returned. Available parameters include:

            * enabled: A boolean value indicating whether the policy is
              enabled on the cluster.
        :returns: A generator of cluster-policy binding instances.
        """
        cluster_id = resource.Resource._get_id(cluster)
        return self._list(
            _cluster_policy.ClusterPolicy, cluster_id=cluster_id, **query
        )

    def get_cluster_policy(self, cluster_policy, cluster):
        """Get a cluster-policy binding.

        :param cluster_policy:
            The value can be the name or ID of a policy or a
            :class:`~openstack.clustering.v1.policy.Policy` instance.
        :param cluster: The value can be the name or ID of a cluster or a
            :class:`~openstack.clustering.v1.cluster.Cluster` instance.

        :returns: a cluster-policy binding object.
        :rtype: :class:`~openstack.clustering.v1.cluster_policy.CLusterPolicy`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            cluster-policy binding matching the criteria could be found.
        """
        return self._get(
            _cluster_policy.ClusterPolicy, cluster_policy, cluster_id=cluster
        )

    def create_receiver(self, **attrs):
        """Create a new receiver from attributes.

        :param dict attrs: Keyword arguments that will be used to create a
            :class:`~openstack.clustering.v1.receiver.Receiver`, it is
            comprised of the properties on the Receiver class.

        :returns: The results of receiver creation.
        :rtype: :class:`~openstack.clustering.v1.receiver.Receiver`.
        """
        return self._create(_receiver.Receiver, **attrs)

    def update_receiver(self, receiver, **attrs):
        """Update a receiver.

        :param receiver: The value can be either the name or ID of a receiver
            or a :class:`~openstack.clustering.v1.receiver.Receiver` instance.
        :param attrs: The attributes to update on the receiver parameter.
            Valid attribute names include ``name``, ``action`` and ``params``.
        :returns: The updated receiver.
        :rtype: :class:`~openstack.clustering.v1.receiver.Receiver`
        """
        return self._update(_receiver.Receiver, receiver, **attrs)

    def delete_receiver(self, receiver, ignore_missing=True):
        """Delete a receiver.

        :param receiver: The value can be either the name or ID of a receiver
            or a :class:`~openstack.clustering.v1.receiver.Receiver` instance.
        :param bool ignore_missing: When set to ``False``, an exception
            :class:`~openstack.exceptions.NotFoundException` will be raised
            when the receiver could not be found. When set to ``True``, no
            exception will be raised when attempting to delete a non-existent
            receiver.

        :returns: ``None``
        """
        self._delete(
            _receiver.Receiver, receiver, ignore_missing=ignore_missing
        )

    def find_receiver(self, name_or_id, ignore_missing=True):
        """Find a single receiver.

        :param str name_or_id: The name or ID of a receiver.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.NotFoundException` will be
            raised when the specified receiver does not exist. When
            set to ``True``, None will be returned when attempting to
            find a nonexistent receiver.
        :returns: A receiver object or None.
        :rtype: :class:`~openstack.clustering.v1.receiver.Receiver`
        """
        return self._find(
            _receiver.Receiver, name_or_id, ignore_missing=ignore_missing
        )

    def get_receiver(self, receiver):
        """Get a single receiver.

        :param receiver: The value can be the name or ID of a receiver or a
            :class:`~openstack.clustering.v1.receiver.Receiver` instance.

        :returns: A receiver object.
        :rtype: :class:`~openstack.clustering.v1.receiver.Receiver`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            receiver matching the criteria could be found.
        """
        return self._get(_receiver.Receiver, receiver)

    def receivers(self, **query):
        """Retrieve a generator of receivers.

        :param kwargs query: Optional query parameters for restricting the
            receivers to be returned. Available parameters include:

            * name: The name of a receiver object.
            * type: The type of receiver objects.
            * cluster_id: The ID of the associated cluster.
            * action: The name of the associated action.
            * sort: A list of sorting keys separated by commas. Each sorting
              key can optionally be attached with a sorting direction
              modifier which can be ``asc`` or ``desc``.
            * global_project: A boolean value indicating whether receivers
            *   from all projects will be returned.

        :returns: A generator of receiver instances.
        """
        return self._list(_receiver.Receiver, **query)

    def get_action(self, action):
        """Get a single action.

        :param action: The value can be the name or ID of an action or a
            :class:`~openstack.clustering.v1.action.Action` instance.

        :returns: an action object.
        :rtype: :class:`~openstack.clustering.v1.action.Action`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            action matching the criteria could be found.
        """
        return self._get(_action.Action, action)

    def actions(self, **query):
        """Retrieve a generator of actions.

        :param kwargs query: Optional query parameters to be sent to
            restrict the actions to be returned. Available parameters include:

            * name: name of action for query.
            * target: ID of the target object for which the actions should be
              returned.
            * action: built-in action types for query.
            * sort: A list of sorting keys separated by commas. Each sorting
              key can optionally be attached with a sorting direction
              modifier which can be ``asc`` or ``desc``.
            * limit: Requests a specified size of returned items from the
              query.  Returns a number of items up to the specified limit
              value.
            * marker: Specifies the ID of the last-seen item. Use the limit
              parameter to make an initial limited request and use the ID of
              the last-seen item from the response as the marker parameter
              value in a subsequent limited request.

        :returns: A generator of action instances.
        """
        return self._list(_action.Action, **query)

    def update_action(self, action, **attrs):
        """Update a profile.

        :param action: Either the ID of the action, or an
            instance of :class:`~openstack.clustering.v1.action.Action`.
        :param attrs: The attributes to update on the action represented by
            the ``value`` parameter.

        :returns: The updated action.
        :rtype: :class:`~openstack.clustering.v1.action.Action`
        """
        return self._update(_action.Action, action, **attrs)

    def get_event(self, event):
        """Get a single event.

        :param event: The value can be the name or ID of an event or a
            :class:`~openstack.clustering.v1.event.Event` instance.

        :returns: an event object.
        :rtype: :class:`~openstack.clustering.v1.event.Event`
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            event matching the criteria could be found.
        """
        return self._get(_event.Event, event)

    def events(self, **query):
        """Retrieve a generator of events.

        :param kwargs query: Optional query parameters to be sent to
            restrict the events to be returned. Available parameters include:

            * obj_name: name string of the object associated with an event.
            * obj_type: type string of the object related to an event. The
              value can be ``cluster``, ``node``, ``policy`` etc.
            * obj_id: ID of the object associated with an event.
            * cluster_id: ID of the cluster associated with the event, if any.
            * action: name of the action associated with an event.
            * sort: A list of sorting keys separated by commas. Each sorting
              key can optionally be attached with a sorting direction
              modifier which can be ``asc`` or ``desc``.
            * limit: Requests a specified size of returned items from the
              query.  Returns a number of items up to the specified limit
              value.
            * marker: Specifies the ID of the last-seen item. Use the limit
              parameter to make an initial limited request and use the ID of
              the last-seen item from the response as the marker parameter
              value in a subsequent limited request.
            * global_project: A boolean specifying whether events from all
              projects should be returned. This option is subject to access
              control checking.

        :returns: A generator of event instances.
        """
        return self._list(_event.Event, **query)

    def services(self, **query):
        """Get a generator of services.

        :returns: A generator of objects that are of type
            :class:`~openstack.clustering.v1.service.Service`
        """
        return self._list(_service.Service, **query)

    def list_profile_type_operations(self, profile_type):
        """Get the operation about a profile type.

        :param profile_type: The name of the profile_type to retrieve or an
            object of
            :class:`~openstack.clustering.v1.profile_type.ProfileType`.

        :returns: A :class:`~openstack.clustering.v1.profile_type.ProfileType`
            object.
        :raises: :class:`~openstack.exceptions.NotFoundException` when no
            profile_type matching the name could be found.
        """
        obj = self._get_resource(_profile_type.ProfileType, profile_type)
        return obj.type_ops(self)

    # ========== Utilities ==========

    def wait_for_status(
        self,
        res: resource.ResourceT,
        status: str,
        failures: list[str] | None = None,
        interval: int | float | None = 2,
        wait: int | None = None,
        attribute: str = 'status',
        callback: ty.Callable[[int], None] | None = None,
    ) -> resource.ResourceT:
        """Wait for the resource to be in a particular status.

        :param session: The session to use for making this request.
        :param resource: The resource to wait on to reach the status. The
            resource must have a status attribute specified via ``attribute``.
        :param status: Desired status of the resource.
        :param failures: Statuses that would indicate the transition
            failed such as 'ERROR'. Defaults to ['ERROR'].
        :param interval: Number of seconds to wait between checks.
        :param wait: Maximum number of seconds to wait for transition.
            Set to ``None`` to wait forever.
        :param attribute: Name of the resource attribute that contains the
            status.
        :param callback: A callback function. This will be called with a single
            value, progress. This is API specific but is generally a percentage
            value from 0-100.

        :return: The updated resource.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
            transition to status failed to occur in ``wait`` seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
            transitioned to one of the states in ``failures``.
        :raises: :class:`~AttributeError` if the resource does not have a
            ``status`` attribute
        """
        return resource.wait_for_status(
            self, res, status, failures, interval, wait, attribute, callback
        )

    def wait_for_delete(
        self,
        res: resource.ResourceT,
        interval: int = 2,
        wait: int = 120,
        callback: ty.Callable[[int], None] | None = None,
    ) -> resource.ResourceT:
        """Wait for a resource to be deleted.

        :param res: The resource to wait on to be deleted.
        :param interval: Number of seconds to wait before to consecutive
            checks.
        :param wait: Maximum number of seconds to wait before the change.
        :param callback: A callback function. This will be called with a single
            value, progress, which is a percentage value from 0-100.

        :returns: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
            to delete failed to occur in the specified seconds.
        """
        return resource.wait_for_delete(self, res, interval, wait, callback)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class MeteringLabelRule(resource.Resource):
    resource_key = 'metering_label_rule'
    resources_key = 'metering_label_rules'
    base_path = '/metering/metering-label-rules'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'direction',
        'metering_label_id',
        'remote_ip_prefix',
        'source_ip_prefix',
        'destination_ip_prefix',
        'project_id',
        'sort_key',
        'sort_dir',
    )

    # Properties
    #: ingress or egress: The direction in which metering label rule is
    #: applied. Default: ``"ingress"``
    direction = resource.Body('direction')
    #: Specify whether the ``remote_ip_prefix`` will be excluded or not
    #: from traffic counters of the metering label, ie: to not count the
    #: traffic of a specific IP address of a range. Default: ``False``,
    #: *Type: bool*
    is_excluded = resource.Body('excluded', type=bool)
    #: The metering label ID to associate with this metering label rule.
    metering_label_id = resource.Body('metering_label_id')
    #: The ID of the project this metering label rule is associated with.
    project_id = resource.Body('project_id', alias='tenant_id')
    #: Tenant_id (deprecated attribute).
    tenant_id = resource.Body('tenant_id', deprecated=True)
    #: The remote IP prefix to be associated with this metering label rule.
    remote_ip_prefix = resource.Body(
        'remote_ip_prefix',
        deprecated=True,
        deprecation_reason="The use of 'remote_ip_prefix' in metering label "
        "rules is deprecated and will be removed in future "
        "releases. One should use instead, the "
        "'source_ip_prefix' and/or 'destination_ip_prefix' "
        "parameters. For more details, you can check the "
        "spec: https://review.opendev.org/#/c/744702/.",
    )

    #: The source IP prefix to be associated with this metering label rule.
    source_ip_prefix = resource.Body('source_ip_prefix')
    #: The destination IP prefix to be associated with this metering label rule
    destination_ip_prefix = resource.Body('destination_ip_prefix')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.functional import base


class TestService(base.BaseFunctionalTest):
    def test_service(self):
        # list all services
        services = list(self.operator_cloud.compute.services())
        self.assertIsNotNone(services)

        # find a service
        self.operator_cloud.compute.find_service(
            services[0].name, host=services[0].host, ignore_missing=False
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Capabilities(resource.Resource):
    base_path = "/capabilities"

    # Capabilities
    allow_fetch = True

    #: Properties
    #: The capabilities description
    description = resource.Body("description")
    #: The name of volume backend capabilities.
    display_name = resource.Body("display_name")
    #: The driver version.
    driver_version = resource.Body("driver_version")
    #: The storage namespace, such as OS::Storage::Capabilities::foo.
    namespace = resource.Body("namespace")
    #: The name of the storage pool.
    pool_name = resource.Body("pool_name")
    #: The backend volume capabilites list, which consists of cinder
    #: standard capabilities and vendor unique properties.
    properties = resource.Body("properties", type=dict)
    #: A list of volume backends used to replicate volumes on this backend.
    replication_targets = resource.Body("replication_targets", type=list)
    #: The storage backend for the backend volume.
    storage_protocol = resource.Body("storage_protocol")
    #: The name of the vendor.
    vendor_name = resource.Body("vendor_name")
    #: The volume type access.
    visibility = resource.Body("visibility")
    #: The name of the back-end volume.
    volume_backend_name = resource.Body("volume_backend_name")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import volume as _volume
from openstack.tests.functional.block_storage.v3 import base


class TestAttachment(base.BaseBlockStorageTest):
    """Test class for volume attachment operations.

    We have implemented a test that performs attachment create
    and attachment delete operations. Attachment create requires
    the instance ID and the volume ID for which we have created a
    volume resource and an instance resource.
    We haven't implemented attachment update test since it requires
    the host connector information which is not readily available to
    us and hard to retrieve. Without passing this information, the
    attachment update operation will fail.
    Similarly, we haven't implement attachment complete test since it
    depends on attachment update and can only be performed when the volume
    status is 'attaching' which is done by attachment update operation.
    """

    def setUp(self):
        super().setUp()

        # Create Volume
        self.volume_name = self.getUniqueString()

        volume = self.user_cloud.block_storage.create_volume(
            name=self.volume_name, size=1
        )
        self.user_cloud.block_storage.wait_for_status(
            volume,
            status='available',
            failures=['error'],
            interval=2,
            wait=self._wait_for_timeout,
        )
        self.assertIsInstance(volume, _volume.Volume)
        self.VOLUME_ID = volume.id

        # Create Server
        self.server_name = self.getUniqueString()
        self.server = self.operator_cloud.compute.create_server(
            name=self.server_name,
            flavor_id=self.flavor.id,
            image_id=self.image.id,
            networks='none',
        )
        self.operator_cloud.compute.wait_for_server(
            self.server, wait=self._wait_for_timeout
        )

    def tearDown(self):
        # Since delete_on_termination flag is set to True, we
        # don't need to cleanup the volume manually
        result = self.operator_cloud.compute.delete_server(self.server.id)
        self.operator_cloud.compute.wait_for_delete(
            self.server, wait=self._wait_for_timeout
        )
        self.assertIsNone(result)
        super().tearDown()

    def test_attachment(self):
        attachment = self.operator_cloud.block_storage.create_attachment(
            self.VOLUME_ID,
            connector={},
            instance_id=self.server.id,
        )
        self.assertIn('id', attachment)
        self.assertIn('status', attachment)
        self.assertIn('instance', attachment)
        self.assertIn('volume_id', attachment)
        self.assertIn('attached_at', attachment)
        self.assertIn('detached_at', attachment)
        self.assertIn('attach_mode', attachment)
        self.assertIn('connection_info', attachment)
        attachment = self.user_cloud.block_storage.delete_attachment(
            attachment.id, ignore_missing=False
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import warnings

from openstack import exceptions
from openstack import resource
from openstack import utils
from openstack import warnings as os_warnings


class Hypervisor(resource.Resource):
    resource_key = 'hypervisor'
    resources_key = 'hypervisors'
    base_path = '/os-hypervisors'

    # capabilities
    allow_fetch = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'hypervisor_hostname_pattern', 'with_servers'
    )

    # Lot of attributes are dropped in 2.88
    _max_microversion = '2.88'

    # Properties
    #: Information about the hypervisor's CPU. Up to 2.28 it was string.
    cpu_info = resource.Body('cpu_info')
    #: IP address of the host
    host_ip = resource.Body('host_ip')
    #: The type of hypervisor
    hypervisor_type = resource.Body('hypervisor_type')
    #: Version of the hypervisor
    hypervisor_version = resource.Body('hypervisor_version')
    #: Name of hypervisor
    name = resource.Body('hypervisor_hostname')
    #: Service details
    service_details = resource.Body('service', type=dict)
    #: List of Servers
    servers = resource.Body('servers', type=list, list_type=dict)
    #: State of hypervisor
    state = resource.Body('state')
    #: Status of hypervisor
    status = resource.Body('status')
    #: The total uptime of the hypervisor and information about average load.
    #: This attribute is set only when querying uptime explicitly.
    uptime = resource.Body('uptime')

    # Attributes deprecated with 2.88
    #: Measurement of the hypervisor's current workload
    current_workload = resource.Body('current_workload', deprecated=True)
    #: Disk space available to the scheduler
    disk_available = resource.Body("disk_available_least", deprecated=True)
    #: The amount, in gigabytes, of local storage used
    local_disk_used = resource.Body('local_gb_used', deprecated=True)
    #: The amount, in gigabytes, of the local storage device
    local_disk_size = resource.Body('local_gb', deprecated=True)
    #: The amount, in gigabytes, of free space on the local storage device
    local_disk_free = resource.Body('free_disk_gb', deprecated=True)
    #: The amount, in megabytes, of memory
    memory_used = resource.Body('memory_mb_used', deprecated=True)
    #: The amount, in megabytes, of total memory
    memory_size = resource.Body('memory_mb', deprecated=True)
    #: The amount, in megabytes, of available memory
    memory_free = resource.Body('free_ram_mb', deprecated=True)
    #: Count of the running virtual machines
    running_vms = resource.Body('running_vms', deprecated=True)
    #: Count of the VCPUs in use
    vcpus_used = resource.Body('vcpus_used', deprecated=True)
    #: Count of all VCPUs
    vcpus = resource.Body('vcpus', deprecated=True)

    def get_uptime(self, session):
        """Get uptime information for the hypervisor

        Updates uptime attribute of the hypervisor object
        """
        warnings.warn(
            "This call is deprecated and is only available until Nova 2.88",
            os_warnings.LegacyAPIWarning,
        )
        if utils.supports_microversion(session, '2.88'):
            raise exceptions.SDKException(
                'Hypervisor.get_uptime is not supported anymore'
            )
        url = utils.urljoin(self.base_path, self.id, 'uptime')
        microversion = self._get_microversion(session)
        response = session.get(url, microversion=microversion)
        self._translate_response(response)
        return self


HypervisorDetail = Hypervisor



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack.message.v2 import _base
from openstack import resource


class Message(_base.MessageResource):
    resources_key = 'messages'
    base_path = '/queues/%(queue_name)s/messages'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True

    _query_mapping = resource.QueryParameters("echo", "include_claimed")

    # Properties
    #: The value in second to specify how long the message has been
    #: posted to the queue.
    age = resource.Body("age")
    #: A dictionary specifies an arbitrary document that constitutes the
    #: body of the message being sent.
    body = resource.Body("body")
    #: An uri string describe the location of the message resource.
    href = resource.Body("href")
    #: The value in seconds to specify how long the server waits before
    #: marking the message as expired and removing it from the queue.
    ttl = resource.Body("ttl")
    #: The name of target queue message is post to or got from.
    queue_name = resource.URI("queue_name")

    # FIXME(stephenfin): This is actually a query arg but we need it for
    # deletions and resource.delete doesn't respect these currently
    claim_id: str | None = None

    def post(self, session, messages):
        request = self._prepare_request(requires_id=False, prepend_key=True)
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id(),
        }
        request.headers.update(headers)
        request.body = {'messages': messages}
        response = session.post(
            request.url, json=request.body, headers=request.headers
        )

        return response.json()['resources']

    def create(self, session, prepend_key=False, base_path=None, **kwargs):
        request = self._prepare_request(
            requires_id=False, prepend_key=prepend_key, base_path=base_path
        )
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id(),
        }
        request.headers.update(headers)
        response = session.post(
            request.url, json=request.body, headers=request.headers
        )

        # For case no message was claimed successfully, 204 No Content
        # message will be returned. In other cases, we translate response
        # body which has `messages` field(list) included.
        if response.status_code != 204:
            self._translate_response(response)

        return self

    def delete(
        self, session, error_message=None, *, microversion=None, **kwargs
    ):
        request = self._prepare_request()
        headers = {
            "Client-ID": self.client_id or str(uuid.uuid4()),
            "X-PROJECT-ID": self.project_id or session.get_project_id(),
        }

        request.headers.update(headers)
        # For Zaqar v2 API requires client to specify claim_id as query
        # parameter when deleting a message that has been claimed, we
        # rebuild the request URI if claim_id is not None.
        if self.claim_id:
            request.url += f'?claim_id={self.claim_id}'
        response = session.delete(request.url, headers=headers)

        self._translate_response(response, has_body=False)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class BgpVpn(resource.Resource):
    resource_key = 'bgpvpn'
    resources_key = 'bgpvpns'
    base_path = '/bgpvpn/bgpvpns'

    _allow_unknown_attrs_in_body = True

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'name',
        'project_id',
        'local_pref',
        'vni',
        'type',
        'networks',
        'routers',
        'ports',
        # NOTE(seba): (route|import|export) targets only support exact matches
        # and have therefore been left out
    )

    # Properties
    #: The Id of the BGPVPN
    id = resource.Body('id')
    #: The BGPVPN's name.
    name = resource.Body('name')
    #: The ID of the project that owns the BGPVPN
    project_id = resource.Body('project_id', alias='tenant_id')
    #: Tenant_id (deprecated attribute).
    tenant_id = resource.Body('tenant_id', deprecated=True)
    #: List of route distinguisher strings.
    route_distinguishers = resource.Body('route_distinguishers')
    #: Route Targets that will be both imported and used for export.
    route_targets = resource.Body('route_targets')
    #: Additional Route Targets that will be imported.
    import_targets = resource.Body('import_targets')
    #: Additional Route Targets that will be used for export.
    export_targets = resource.Body('export_targets')
    #: The default BGP LOCAL_PREF of routes that will be advertised to
    #: the BGPVPN.
    local_pref = resource.Body('local_pref')
    #: The globally-assigned VXLAN vni for the BGP VPN.
    vni = resource.Body('vni')
    #: Selection of the type of VPN and the technology behind it.
    #: Allowed values are l2 or l3.
    type = resource.Body('type')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Member(resource.Resource):
    resources_key = 'members'
    base_path = '/images/%(image_id)s/members'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # See https://bugs.launchpad.net/glance/+bug/1526991 for member/member_id
    # 'member' is documented incorrectly as being deprecated but it's the
    # only thing that works. 'member_id' is not accepted.

    #: The ID of the image member. An image member is a tenant
    #: with whom the image is shared.
    member_id = resource.Body('member', alternate_id=True)
    #: The date and time when the member was created.
    created_at = resource.Body('created_at')
    #: Image ID stored through the image API. Typically a UUID.
    image_id = resource.URI('image_id')
    #: The status of the image.
    status = resource.Body('status')
    #: The URL for schema of the member.
    schema = resource.Body('schema')
    #: The date and time when the member was updated.
    updated_at = resource.Body('updated_at')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.load_balancer.v2 import availability_zone
from openstack.load_balancer.v2 import availability_zone_profile
from openstack.load_balancer.v2 import flavor
from openstack.load_balancer.v2 import flavor_profile
from openstack.load_balancer.v2 import health_monitor
from openstack.load_balancer.v2 import l7_policy
from openstack.load_balancer.v2 import l7_rule
from openstack.load_balancer.v2 import listener
from openstack.load_balancer.v2 import load_balancer
from openstack.load_balancer.v2 import member
from openstack.load_balancer.v2 import pool
from openstack.load_balancer.v2 import quota
from openstack.tests.functional import base


class TestLoadBalancer(base.BaseFunctionalTest):
    HM_ID = None
    L7POLICY_ID = None
    LB_ID = None
    LISTENER_ID = None
    MEMBER_ID = None
    POOL_ID = None
    VIP_SUBNET_ID = None
    PROJECT_ID = None
    FLAVOR_PROFILE_ID = None
    FLAVOR_ID = None
    AVAILABILITY_ZONE_PROFILE_ID = None
    AMPHORA_ID = None
    PROTOCOL = 'HTTP'
    PROTOCOL_PORT = 80
    LB_ALGORITHM = 'ROUND_ROBIN'
    MEMBER_ADDRESS = '192.0.2.16'
    WEIGHT = 10
    DELAY = 2
    TIMEOUT = 1
    MAX_RETRY = 3
    HM_TYPE = 'HTTP'
    ACTION = 'REDIRECT_TO_URL'
    REDIRECT_URL = 'http://www.example.com'
    COMPARE_TYPE = 'CONTAINS'
    L7RULE_TYPE = 'HOST_NAME'
    L7RULE_VALUE = 'example'
    AMPHORA = 'amphora'
    FLAVOR_DATA = '{"loadbalancer_topology": "SINGLE"}'
    AVAILABILITY_ZONE_DATA = '{"compute_zone": "nova"}'
    DESCRIPTION = 'Test description'

    _wait_for_timeout_key = 'OPENSTACKSDK_FUNC_TEST_TIMEOUT_LOAD_BALANCER'

    # TODO(shade): Creating load balancers can be slow on some hosts due to
    #              nova instance boot times (up to ten minutes). This used to
    #              use setUpClass, but that's a whole other pile of bad, so
    #              we may need to engineer something pleasing here.
    def setUp(self):
        super().setUp()
        self.require_service('load-balancer')

        self.HM_NAME = self.getUniqueString()
        self.L7POLICY_NAME = self.getUniqueString()
        self.LB_NAME = self.getUniqueString()
        self.LISTENER_NAME = self.getUniqueString()
        self.MEMBER_NAME = self.getUniqueString()
        self.POOL_NAME = self.getUniqueString()
        self.UPDATE_NAME = self.getUniqueString()
        self.UPDATE_DESCRIPTION = self.getUniqueString()
        self.FLAVOR_PROFILE_NAME = self.getUniqueString()
        self.FLAVOR_NAME = self.getUniqueString()
        self.AVAILABILITY_ZONE_PROFILE_NAME = self.getUniqueString()
        self.AVAILABILITY_ZONE_NAME = self.getUniqueString()
        subnets = list(self.operator_cloud.network.subnets())
        self.VIP_SUBNET_ID = subnets[0].id
        self.PROJECT_ID = self.operator_cloud.session.get_project_id()
        test_quota = self.operator_cloud.load_balancer.update_quota(
            self.PROJECT_ID,
            **{
                'load_balancer': 100,
                'pool': 100,
                'listener': 100,
                'health_monitor': 100,
                'member': 100,
            },
        )
        assert isinstance(test_quota, quota.Quota)
        self.assertEqual(self.PROJECT_ID, test_quota.id)

        test_flavor_profile = (
            self.operator_cloud.load_balancer.create_flavor_profile(
                name=self.FLAVOR_PROFILE_NAME,
                provider_name=self.AMPHORA,
                flavor_data=self.FLAVOR_DATA,
            )
        )
        assert isinstance(test_flavor_profile, flavor_profile.FlavorProfile)
        self.assertEqual(self.FLAVOR_PROFILE_NAME, test_flavor_profile.name)
        self.FLAVOR_PROFILE_ID = test_flavor_profile.id

        test_flavor = self.operator_cloud.load_balancer.create_flavor(
            name=self.FLAVOR_NAME,
            flavor_profile_id=self.FLAVOR_PROFILE_ID,
            is_enabled=True,
            description=self.DESCRIPTION,
        )
        assert isinstance(test_flavor, flavor.Flavor)
        self.assertEqual(self.FLAVOR_NAME, test_flavor.name)
        self.FLAVOR_ID = test_flavor.id

        test_az_profile = (
            self.operator_cloud.load_balancer.create_availability_zone_profile(
                name=self.AVAILABILITY_ZONE_PROFILE_NAME,
                provider_name=self.AMPHORA,
                availability_zone_data=self.AVAILABILITY_ZONE_DATA,
            )
        )
        assert isinstance(
            test_az_profile, availability_zone_profile.AvailabilityZoneProfile
        )
        self.assertEqual(
            self.AVAILABILITY_ZONE_PROFILE_NAME, test_az_profile.name
        )
        self.AVAILABILITY_ZONE_PROFILE_ID = test_az_profile.id

        test_az = self.operator_cloud.load_balancer.create_availability_zone(
            name=self.AVAILABILITY_ZONE_NAME,
            availability_zone_profile_id=self.AVAILABILITY_ZONE_PROFILE_ID,
            is_enabled=True,
            description=self.DESCRIPTION,
        )
        assert isinstance(test_az, availability_zone.AvailabilityZone)
        self.assertEqual(self.AVAILABILITY_ZONE_NAME, test_az.name)

        test_lb = self.operator_cloud.load_balancer.create_load_balancer(
            name=self.LB_NAME,
            vip_subnet_id=self.VIP_SUBNET_ID,
            project_id=self.PROJECT_ID,
        )
        assert isinstance(test_lb, load_balancer.LoadBalancer)
        self.assertEqual(self.LB_NAME, test_lb.name)
        # Wait for the LB to go ACTIVE.  On non-virtualization enabled hosts
        # it can take nova up to ten minutes to boot a VM.
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            test_lb.id, interval=1, wait=self._wait_for_timeout
        )
        self.LB_ID = test_lb.id

        amphorae = self.operator_cloud.load_balancer.amphorae(
            loadbalancer_id=self.LB_ID
        )
        for amp in amphorae:
            self.AMPHORA_ID = amp.id

        test_listener = self.operator_cloud.load_balancer.create_listener(
            name=self.LISTENER_NAME,
            protocol=self.PROTOCOL,
            protocol_port=self.PROTOCOL_PORT,
            loadbalancer_id=self.LB_ID,
        )
        assert isinstance(test_listener, listener.Listener)
        self.assertEqual(self.LISTENER_NAME, test_listener.name)
        self.LISTENER_ID = test_listener.id
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        test_pool = self.operator_cloud.load_balancer.create_pool(
            name=self.POOL_NAME,
            protocol=self.PROTOCOL,
            lb_algorithm=self.LB_ALGORITHM,
            listener_id=self.LISTENER_ID,
        )
        assert isinstance(test_pool, pool.Pool)
        self.assertEqual(self.POOL_NAME, test_pool.name)
        self.POOL_ID = test_pool.id
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        test_member = self.operator_cloud.load_balancer.create_member(
            pool=self.POOL_ID,
            name=self.MEMBER_NAME,
            address=self.MEMBER_ADDRESS,
            protocol_port=self.PROTOCOL_PORT,
            weight=self.WEIGHT,
        )
        assert isinstance(test_member, member.Member)
        self.assertEqual(self.MEMBER_NAME, test_member.name)
        self.MEMBER_ID = test_member.id
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        test_hm = self.operator_cloud.load_balancer.create_health_monitor(
            pool_id=self.POOL_ID,
            name=self.HM_NAME,
            delay=self.DELAY,
            timeout=self.TIMEOUT,
            max_retries=self.MAX_RETRY,
            type=self.HM_TYPE,
        )
        assert isinstance(test_hm, health_monitor.HealthMonitor)
        self.assertEqual(self.HM_NAME, test_hm.name)
        self.HM_ID = test_hm.id
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        test_l7policy = self.operator_cloud.load_balancer.create_l7_policy(
            listener_id=self.LISTENER_ID,
            name=self.L7POLICY_NAME,
            action=self.ACTION,
            redirect_url=self.REDIRECT_URL,
        )
        assert isinstance(test_l7policy, l7_policy.L7Policy)
        self.assertEqual(self.L7POLICY_NAME, test_l7policy.name)
        self.L7POLICY_ID = test_l7policy.id
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        test_l7rule = self.operator_cloud.load_balancer.create_l7_rule(
            l7_policy=self.L7POLICY_ID,
            compare_type=self.COMPARE_TYPE,
            type=self.L7RULE_TYPE,
            value=self.L7RULE_VALUE,
        )
        assert isinstance(test_l7rule, l7_rule.L7Rule)
        self.assertEqual(self.COMPARE_TYPE, test_l7rule.compare_type)
        self.L7RULE_ID = test_l7rule.id
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

    def tearDown(self):
        self.operator_cloud.load_balancer.get_load_balancer(self.LB_ID)
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        self.operator_cloud.load_balancer.delete_quota(
            self.PROJECT_ID, ignore_missing=False
        )

        self.operator_cloud.load_balancer.delete_l7_rule(
            self.L7RULE_ID, l7_policy=self.L7POLICY_ID, ignore_missing=False
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        self.operator_cloud.load_balancer.delete_l7_policy(
            self.L7POLICY_ID, ignore_missing=False
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        self.operator_cloud.load_balancer.delete_health_monitor(
            self.HM_ID, ignore_missing=False
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        self.operator_cloud.load_balancer.delete_member(
            self.MEMBER_ID, self.POOL_ID, ignore_missing=False
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        self.operator_cloud.load_balancer.delete_pool(
            self.POOL_ID, ignore_missing=False
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        self.operator_cloud.load_balancer.delete_listener(
            self.LISTENER_ID, ignore_missing=False
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )

        self.operator_cloud.load_balancer.delete_load_balancer(
            self.LB_ID, ignore_missing=False
        )
        super().tearDown()

        self.operator_cloud.load_balancer.delete_flavor(
            self.FLAVOR_ID, ignore_missing=False
        )

        self.operator_cloud.load_balancer.delete_flavor_profile(
            self.FLAVOR_PROFILE_ID, ignore_missing=False
        )

        self.operator_cloud.load_balancer.delete_availability_zone(
            self.AVAILABILITY_ZONE_NAME, ignore_missing=False
        )

        self.operator_cloud.load_balancer.delete_availability_zone_profile(
            self.AVAILABILITY_ZONE_PROFILE_ID, ignore_missing=False
        )

    def test_lb_find(self):
        test_lb = self.operator_cloud.load_balancer.find_load_balancer(
            self.LB_NAME
        )
        self.assertEqual(self.LB_ID, test_lb.id)

    def test_lb_get(self):
        test_lb = self.operator_cloud.load_balancer.get_load_balancer(
            self.LB_ID
        )
        self.assertEqual(self.LB_NAME, test_lb.name)
        self.assertEqual(self.LB_ID, test_lb.id)
        self.assertEqual(self.VIP_SUBNET_ID, test_lb.vip_subnet_id)

    def test_lb_get_stats(self):
        test_lb_stats = (
            self.operator_cloud.load_balancer.get_load_balancer_statistics(
                self.LB_ID
            )
        )
        self.assertEqual(0, test_lb_stats.active_connections)
        self.assertEqual(0, test_lb_stats.bytes_in)
        self.assertEqual(0, test_lb_stats.bytes_out)
        self.assertEqual(0, test_lb_stats.request_errors)
        self.assertEqual(0, test_lb_stats.total_connections)

    def test_lb_list(self):
        names = [
            lb.name
            for lb in self.operator_cloud.load_balancer.load_balancers()
        ]
        self.assertIn(self.LB_NAME, names)

    def test_lb_update(self):
        self.operator_cloud.load_balancer.update_load_balancer(
            self.LB_ID, name=self.UPDATE_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_lb = self.operator_cloud.load_balancer.get_load_balancer(
            self.LB_ID
        )
        self.assertEqual(self.UPDATE_NAME, test_lb.name)

        self.operator_cloud.load_balancer.update_load_balancer(
            self.LB_ID, name=self.LB_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_lb = self.operator_cloud.load_balancer.get_load_balancer(
            self.LB_ID
        )
        self.assertEqual(self.LB_NAME, test_lb.name)

    def test_lb_failover(self):
        self.operator_cloud.load_balancer.failover_load_balancer(self.LB_ID)
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_lb = self.operator_cloud.load_balancer.get_load_balancer(
            self.LB_ID
        )
        self.assertEqual(self.LB_NAME, test_lb.name)

    def test_listener_find(self):
        test_listener = self.operator_cloud.load_balancer.find_listener(
            self.LISTENER_NAME
        )
        self.assertEqual(self.LISTENER_ID, test_listener.id)

    def test_listener_get(self):
        test_listener = self.operator_cloud.load_balancer.get_listener(
            self.LISTENER_ID
        )
        self.assertEqual(self.LISTENER_NAME, test_listener.name)
        self.assertEqual(self.LISTENER_ID, test_listener.id)
        self.assertEqual(self.PROTOCOL, test_listener.protocol)
        self.assertEqual(self.PROTOCOL_PORT, test_listener.protocol_port)

    def test_listener_get_stats(self):
        test_listener_stats = (
            self.operator_cloud.load_balancer.get_listener_statistics(
                self.LISTENER_ID
            )
        )
        self.assertEqual(0, test_listener_stats.active_connections)
        self.assertEqual(0, test_listener_stats.bytes_in)
        self.assertEqual(0, test_listener_stats.bytes_out)
        self.assertEqual(0, test_listener_stats.request_errors)
        self.assertEqual(0, test_listener_stats.total_connections)

    def test_listener_list(self):
        names = [
            ls.name for ls in self.operator_cloud.load_balancer.listeners()
        ]
        self.assertIn(self.LISTENER_NAME, names)

    def test_listener_update(self):
        self.operator_cloud.load_balancer.get_load_balancer(self.LB_ID)

        self.operator_cloud.load_balancer.update_listener(
            self.LISTENER_ID, name=self.UPDATE_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_listener = self.operator_cloud.load_balancer.get_listener(
            self.LISTENER_ID
        )
        self.assertEqual(self.UPDATE_NAME, test_listener.name)

        self.operator_cloud.load_balancer.update_listener(
            self.LISTENER_ID, name=self.LISTENER_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_listener = self.operator_cloud.load_balancer.get_listener(
            self.LISTENER_ID
        )
        self.assertEqual(self.LISTENER_NAME, test_listener.name)

    def test_pool_find(self):
        test_pool = self.operator_cloud.load_balancer.find_pool(self.POOL_NAME)
        self.assertEqual(self.POOL_ID, test_pool.id)

    def test_pool_get(self):
        test_pool = self.operator_cloud.load_balancer.get_pool(self.POOL_ID)
        self.assertEqual(self.POOL_NAME, test_pool.name)
        self.assertEqual(self.POOL_ID, test_pool.id)
        self.assertEqual(self.PROTOCOL, test_pool.protocol)

    def test_pool_list(self):
        names = [
            pool.name for pool in self.operator_cloud.load_balancer.pools()
        ]
        self.assertIn(self.POOL_NAME, names)

    def test_pool_update(self):
        self.operator_cloud.load_balancer.get_load_balancer(self.LB_ID)

        self.operator_cloud.load_balancer.update_pool(
            self.POOL_ID, name=self.UPDATE_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_pool = self.operator_cloud.load_balancer.get_pool(self.POOL_ID)
        self.assertEqual(self.UPDATE_NAME, test_pool.name)

        self.operator_cloud.load_balancer.update_pool(
            self.POOL_ID, name=self.POOL_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_pool = self.operator_cloud.load_balancer.get_pool(self.POOL_ID)
        self.assertEqual(self.POOL_NAME, test_pool.name)

    def test_member_find(self):
        test_member = self.operator_cloud.load_balancer.find_member(
            self.MEMBER_NAME, self.POOL_ID
        )
        self.assertEqual(self.MEMBER_ID, test_member.id)

    def test_member_get(self):
        test_member = self.operator_cloud.load_balancer.get_member(
            self.MEMBER_ID, self.POOL_ID
        )
        self.assertEqual(self.MEMBER_NAME, test_member.name)
        self.assertEqual(self.MEMBER_ID, test_member.id)
        self.assertEqual(self.MEMBER_ADDRESS, test_member.address)
        self.assertEqual(self.PROTOCOL_PORT, test_member.protocol_port)
        self.assertEqual(self.WEIGHT, test_member.weight)

    def test_member_list(self):
        names = [
            mb.name
            for mb in self.operator_cloud.load_balancer.members(self.POOL_ID)
        ]
        self.assertIn(self.MEMBER_NAME, names)

    def test_member_update(self):
        self.operator_cloud.load_balancer.get_load_balancer(self.LB_ID)

        self.operator_cloud.load_balancer.update_member(
            self.MEMBER_ID, self.POOL_ID, name=self.UPDATE_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_member = self.operator_cloud.load_balancer.get_member(
            self.MEMBER_ID, self.POOL_ID
        )
        self.assertEqual(self.UPDATE_NAME, test_member.name)

        self.operator_cloud.load_balancer.update_member(
            self.MEMBER_ID, self.POOL_ID, name=self.MEMBER_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_member = self.operator_cloud.load_balancer.get_member(
            self.MEMBER_ID, self.POOL_ID
        )
        self.assertEqual(self.MEMBER_NAME, test_member.name)

    def test_health_monitor_find(self):
        test_hm = self.operator_cloud.load_balancer.find_health_monitor(
            self.HM_NAME
        )
        self.assertEqual(self.HM_ID, test_hm.id)

    def test_health_monitor_get(self):
        test_hm = self.operator_cloud.load_balancer.get_health_monitor(
            self.HM_ID
        )
        self.assertEqual(self.HM_NAME, test_hm.name)
        self.assertEqual(self.HM_ID, test_hm.id)
        self.assertEqual(self.DELAY, test_hm.delay)
        self.assertEqual(self.TIMEOUT, test_hm.timeout)
        self.assertEqual(self.MAX_RETRY, test_hm.max_retries)
        self.assertEqual(self.HM_TYPE, test_hm.type)

    def test_health_monitor_list(self):
        names = [
            hm.name
            for hm in self.operator_cloud.load_balancer.health_monitors()
        ]
        self.assertIn(self.HM_NAME, names)

    def test_health_monitor_update(self):
        self.operator_cloud.load_balancer.get_load_balancer(self.LB_ID)

        self.operator_cloud.load_balancer.update_health_monitor(
            self.HM_ID, name=self.UPDATE_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_hm = self.operator_cloud.load_balancer.get_health_monitor(
            self.HM_ID
        )
        self.assertEqual(self.UPDATE_NAME, test_hm.name)

        self.operator_cloud.load_balancer.update_health_monitor(
            self.HM_ID, name=self.HM_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_hm = self.operator_cloud.load_balancer.get_health_monitor(
            self.HM_ID
        )
        self.assertEqual(self.HM_NAME, test_hm.name)

    def test_l7_policy_find(self):
        test_l7_policy = self.operator_cloud.load_balancer.find_l7_policy(
            self.L7POLICY_NAME
        )
        self.assertEqual(self.L7POLICY_ID, test_l7_policy.id)

    def test_l7_policy_get(self):
        test_l7_policy = self.operator_cloud.load_balancer.get_l7_policy(
            self.L7POLICY_ID
        )
        self.assertEqual(self.L7POLICY_NAME, test_l7_policy.name)
        self.assertEqual(self.L7POLICY_ID, test_l7_policy.id)
        self.assertEqual(self.ACTION, test_l7_policy.action)

    def test_l7_policy_list(self):
        names = [
            l7.name for l7 in self.operator_cloud.load_balancer.l7_policies()
        ]
        self.assertIn(self.L7POLICY_NAME, names)

    def test_l7_policy_update(self):
        self.operator_cloud.load_balancer.get_load_balancer(self.LB_ID)

        self.operator_cloud.load_balancer.update_l7_policy(
            self.L7POLICY_ID, name=self.UPDATE_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_l7_policy = self.operator_cloud.load_balancer.get_l7_policy(
            self.L7POLICY_ID
        )
        self.assertEqual(self.UPDATE_NAME, test_l7_policy.name)

        self.operator_cloud.load_balancer.update_l7_policy(
            self.L7POLICY_ID, name=self.L7POLICY_NAME
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_l7_policy = self.operator_cloud.load_balancer.get_l7_policy(
            self.L7POLICY_ID
        )
        self.assertEqual(self.L7POLICY_NAME, test_l7_policy.name)

    def test_l7_rule_find(self):
        test_l7_rule = self.operator_cloud.load_balancer.find_l7_rule(
            self.L7RULE_ID, self.L7POLICY_ID
        )
        self.assertEqual(self.L7RULE_ID, test_l7_rule.id)
        self.assertEqual(self.L7RULE_TYPE, test_l7_rule.type)

    def test_l7_rule_get(self):
        test_l7_rule = self.operator_cloud.load_balancer.get_l7_rule(
            self.L7RULE_ID, l7_policy=self.L7POLICY_ID
        )
        self.assertEqual(self.L7RULE_ID, test_l7_rule.id)
        self.assertEqual(self.COMPARE_TYPE, test_l7_rule.compare_type)
        self.assertEqual(self.L7RULE_TYPE, test_l7_rule.type)
        self.assertEqual(self.L7RULE_VALUE, test_l7_rule.rule_value)

    def test_l7_rule_list(self):
        ids = [
            l7.id
            for l7 in self.operator_cloud.load_balancer.l7_rules(
                l7_policy=self.L7POLICY_ID
            )
        ]
        self.assertIn(self.L7RULE_ID, ids)

    def test_l7_rule_update(self):
        self.operator_cloud.load_balancer.get_load_balancer(self.LB_ID)

        self.operator_cloud.load_balancer.update_l7_rule(
            self.L7RULE_ID,
            l7_policy=self.L7POLICY_ID,
            rule_value=self.UPDATE_NAME,
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_l7_rule = self.operator_cloud.load_balancer.get_l7_rule(
            self.L7RULE_ID, l7_policy=self.L7POLICY_ID
        )
        self.assertEqual(self.UPDATE_NAME, test_l7_rule.rule_value)

        self.operator_cloud.load_balancer.update_l7_rule(
            self.L7RULE_ID,
            l7_policy=self.L7POLICY_ID,
            rule_value=self.L7RULE_VALUE,
        )
        self.operator_cloud.load_balancer.wait_for_load_balancer(
            self.LB_ID, wait=self._wait_for_timeout
        )
        test_l7_rule = self.operator_cloud.load_balancer.get_l7_rule(
            self.L7RULE_ID,
            l7_policy=self.L7POLICY_ID,
        )
        self.assertEqual(self.L7RULE_VALUE, test_l7_rule.rule_value)

    def test_quota_list(self):
        for qot in self.operator_cloud.load_balancer.quotas():
            self.assertIsNotNone(qot.project_id)

    def test_quota_get(self):
        test_quota = self.operator_cloud.load_balancer.get_quota(
            self.PROJECT_ID
        )
        self.assertEqual(self.PROJECT_ID, test_quota.id)

    def test_quota_update(self):
        attrs = {'load_balancer': 12345, 'pool': 67890}
        for project_quota in self.operator_cloud.load_balancer.quotas():
            self.operator_cloud.load_balancer.update_quota(
                project_quota, **attrs
            )
            new_quota = self.operator_cloud.load_balancer.get_quota(
                project_quota.project_id
            )
            self.assertEqual(12345, new_quota.load_balancers)
            self.assertEqual(67890, new_quota.pools)

    def test_default_quota(self):
        self.operator_cloud.load_balancer.get_quota_default()

    def test_providers(self):
        providers = self.operator_cloud.load_balancer.providers()
        # Make sure our default provider is in the list
        self.assertTrue(
            any(prov['name'] == self.AMPHORA for prov in providers)
        )

    def test_provider_flavor_capabilities(self):
        capabilities = (
            self.operator_cloud.load_balancer.provider_flavor_capabilities(
                self.AMPHORA
            )
        )
        # Make sure a known capability is in the default provider
        self.assertTrue(
            any(cap['name'] == 'loadbalancer_topology' for cap in capabilities)
        )

    def test_flavor_profile_find(self):
        test_profile = self.operator_cloud.load_balancer.find_flavor_profile(
            self.FLAVOR_PROFILE_NAME
        )
        self.assertEqual(self.FLAVOR_PROFILE_ID, test_profile.id)

    def test_flavor_profile_get(self):
        test_flavor_profile = (
            self.operator_cloud.load_balancer.get_flavor_profile(
                self.FLAVOR_PROFILE_ID
            )
        )
        self.assertEqual(self.FLAVOR_PROFILE_NAME, test_flavor_profile.name)
        self.assertEqual(self.FLAVOR_PROFILE_ID, test_flavor_profile.id)
        self.assertEqual(self.AMPHORA, test_flavor_profile.provider_name)
        self.assertEqual(self.FLAVOR_DATA, test_flavor_profile.flavor_data)

    def test_flavor_profile_list(self):
        names = [
            fv.name
            for fv in self.operator_cloud.load_balancer.flavor_profiles()
        ]
        self.assertIn(self.FLAVOR_PROFILE_NAME, names)

    def test_flavor_profile_update(self):
        self.operator_cloud.load_balancer.update_flavor_profile(
            self.FLAVOR_PROFILE_ID, name=self.UPDATE_NAME
        )
        test_flavor_profile = (
            self.operator_cloud.load_balancer.get_flavor_profile(
                self.FLAVOR_PROFILE_ID
            )
        )
        self.assertEqual(self.UPDATE_NAME, test_flavor_profile.name)

        self.operator_cloud.load_balancer.update_flavor_profile(
            self.FLAVOR_PROFILE_ID, name=self.FLAVOR_PROFILE_NAME
        )
        test_flavor_profile = (
            self.operator_cloud.load_balancer.get_flavor_profile(
                self.FLAVOR_PROFILE_ID
            )
        )
        self.assertEqual(self.FLAVOR_PROFILE_NAME, test_flavor_profile.name)

    def test_flavor_find(self):
        test_flavor = self.operator_cloud.load_balancer.find_flavor(
            self.FLAVOR_NAME
        )
        self.assertEqual(self.FLAVOR_ID, test_flavor.id)

    def test_flavor_get(self):
        test_flavor = self.operator_cloud.load_balancer.get_flavor(
            self.FLAVOR_ID
        )
        self.assertEqual(self.FLAVOR_NAME, test_flavor.name)
        self.assertEqual(self.FLAVOR_ID, test_flavor.id)
        self.assertEqual(self.DESCRIPTION, test_flavor.description)
        self.assertEqual(self.FLAVOR_PROFILE_ID, test_flavor.flavor_profile_id)

    def test_flavor_list(self):
        names = [fv.name for fv in self.operator_cloud.load_balancer.flavors()]
        self.assertIn(self.FLAVOR_NAME, names)

    def test_flavor_update(self):
        self.operator_cloud.load_balancer.update_flavor(
            self.FLAVOR_ID, name=self.UPDATE_NAME
        )
        test_flavor = self.operator_cloud.load_balancer.get_flavor(
            self.FLAVOR_ID
        )
        self.assertEqual(self.UPDATE_NAME, test_flavor.name)

        self.operator_cloud.load_balancer.update_flavor(
            self.FLAVOR_ID, name=self.FLAVOR_NAME
        )
        test_flavor = self.operator_cloud.load_balancer.get_flavor(
            self.FLAVOR_ID
        )
        self.assertEqual(self.FLAVOR_NAME, test_flavor.name)

    def test_amphora_list(self):
        amp_ids = [
            amp.id for amp in self.operator_cloud.load_balancer.amphorae()
        ]
        self.assertIn(self.AMPHORA_ID, amp_ids)

    def test_amphora_find(self):
        test_amphora = self.operator_cloud.load_balancer.find_amphora(
            self.AMPHORA_ID
        )
        self.assertEqual(self.AMPHORA_ID, test_amphora.id)

    def test_amphora_get(self):
        test_amphora = self.operator_cloud.load_balancer.get_amphora(
            self.AMPHORA_ID
        )
        self.assertEqual(self.AMPHORA_ID, test_amphora.id)

    def test_amphora_configure(self):
        self.operator_cloud.load_balancer.configure_amphora(self.AMPHORA_ID)
        test_amp = self.operator_cloud.load_balancer.get_amphora(
            self.AMPHORA_ID
        )
        self.assertEqual(self.AMPHORA_ID, test_amp.id)

    def test_amphora_failover(self):
        self.operator_cloud.load_balancer.failover_amphora(self.AMPHORA_ID)
        test_amp = self.operator_cloud.load_balancer.get_amphora(
            self.AMPHORA_ID
        )
        self.assertEqual(self.AMPHORA_ID, test_amp.id)

    def test_availability_zone_profile_find(self):
        test_profile = (
            self.operator_cloud.load_balancer.find_availability_zone_profile(
                self.AVAILABILITY_ZONE_PROFILE_NAME
            )
        )
        self.assertEqual(self.AVAILABILITY_ZONE_PROFILE_ID, test_profile.id)

    def test_availability_zone_profile_get(self):
        test_availability_zone_profile = (
            self.operator_cloud.load_balancer.get_availability_zone_profile(
                self.AVAILABILITY_ZONE_PROFILE_ID
            )
        )
        self.assertEqual(
            self.AVAILABILITY_ZONE_PROFILE_NAME,
            test_availability_zone_profile.name,
        )
        self.assertEqual(
            self.AVAILABILITY_ZONE_PROFILE_ID,
            test_availability_zone_profile.id,
        )
        self.assertEqual(
            self.AMPHORA, test_availability_zone_profile.provider_name
        )
        self.assertEqual(
            self.AVAILABILITY_ZONE_DATA,
            test_availability_zone_profile.availability_zone_data,
        )

    def test_availability_zone_profile_list(self):
        names = [
            az.name
            for az in self.operator_cloud.load_balancer.availability_zone_profiles()  # noqa: E501
        ]
        self.assertIn(self.AVAILABILITY_ZONE_PROFILE_NAME, names)

    def test_availability_zone_profile_update(self):
        self.operator_cloud.load_balancer.update_availability_zone_profile(
            self.AVAILABILITY_ZONE_PROFILE_ID, name=self.UPDATE_NAME
        )
        test_availability_zone_profile = (
            self.operator_cloud.load_balancer.get_availability_zone_profile(
                self.AVAILABILITY_ZONE_PROFILE_ID
            )
        )
        self.assertEqual(self.UPDATE_NAME, test_availability_zone_profile.name)

        self.operator_cloud.load_balancer.update_availability_zone_profile(
            self.AVAILABILITY_ZONE_PROFILE_ID,
            name=self.AVAILABILITY_ZONE_PROFILE_NAME,
        )
        test_availability_zone_profile = (
            self.operator_cloud.load_balancer.get_availability_zone_profile(
                self.AVAILABILITY_ZONE_PROFILE_ID
            )
        )
        self.assertEqual(
            self.AVAILABILITY_ZONE_PROFILE_NAME,
            test_availability_zone_profile.name,
        )

    def test_availability_zone_find(self):
        test_availability_zone = (
            self.operator_cloud.load_balancer.find_availability_zone(
                self.AVAILABILITY_ZONE_NAME
            )
        )
        self.assertEqual(
            self.AVAILABILITY_ZONE_NAME, test_availability_zone.name
        )

    def test_availability_zone_get(self):
        test_availability_zone = (
            self.operator_cloud.load_balancer.get_availability_zone(
                self.AVAILABILITY_ZONE_NAME
            )
        )
        self.assertEqual(
            self.AVAILABILITY_ZONE_NAME, test_availability_zone.name
        )
        self.assertEqual(self.DESCRIPTION, test_availability_zone.description)
        self.assertEqual(
            self.AVAILABILITY_ZONE_PROFILE_ID,
            test_availability_zone.availability_zone_profile_id,
        )

    def test_availability_zone_list(self):
        names = [
            az.name
            for az in self.operator_cloud.load_balancer.availability_zones()
        ]
        self.assertIn(self.AVAILABILITY_ZONE_NAME, names)

    def test_availability_zone_update(self):
        self.operator_cloud.load_balancer.update_availability_zone(
            self.AVAILABILITY_ZONE_NAME, description=self.UPDATE_DESCRIPTION
        )
        test_availability_zone = (
            self.operator_cloud.load_balancer.get_availability_zone(
                self.AVAILABILITY_ZONE_NAME
            )
        )
        self.assertEqual(
            self.UPDATE_DESCRIPTION, test_availability_zone.description
        )

        self.operator_cloud.load_balancer.update_availability_zone(
            self.AVAILABILITY_ZONE_NAME, description=self.DESCRIPTION
        )
        test_availability_zone = (
            self.operator_cloud.load_balancer.get_availability_zone(
                self.AVAILABILITY_ZONE_NAME
            )
        )
        self.assertEqual(self.DESCRIPTION, test_availability_zone.description)
